/*--- formatted by Jindent 2.1, (www.c-lab.de/~jindent) ---*/

/*
 * Copyright (c) 1999 / 2000  Johannes Plachy
 *
 * JMailSrv@jps.at
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * If you modify this file, please send us a copy.
 */

/********************************************************************************************
* CVS Info
*
* $Source: /cvsroot/jmailsrv/jmailserver/at/jps/mailserver/AutoresponderController.java,v $
*
* $Author: furykid $
* $Date: 2002/01/20 16:07:13 $
*
* modification history:
*
* $Log: AutoresponderController.java,v $
* Revision 1.2  2002/01/20 16:07:13  furykid
* added new header for cvs information
*

*********************************************************************************************/


package at.jps.mailserver;

import java.io.*;
import java.text.*;
import java.util.*;

/**
 * Class declaration
 * @author
 * @version %I%, %G%
 */
class AutoresponderController implements CFGUpdateable
{
   final private static String            AutoResponderReceiver = "autoresponder";
   private ConfigurationManager           ivConfigurationManager = null;
   private static AutoresponderController cvInstance = null;

   /**
    * Constructor declaration
    */
   private AutoresponderController() {}


   /**
    * Method declaration
    * @return
    */
   final static AutoresponderController getInstance()
   {
      if ( cvInstance == null )
      {
         cvInstance = new AutoresponderController();
      }

      return cvInstance;
   }


   /**
    * Method declaration
    * @return
    */
   final private ConfigurationManager getConfiguration()
   {
      return ivConfigurationManager;
   }


   /**
    * Method declaration
    * @param ConfigurationManager
    */
   final public void updateCFG( ConfigurationManager ConfigurationManager )
   {
      ivConfigurationManager = ConfigurationManager;
   }


   /**
    * Method declaration
    * @return
    */
   final static String getAutoResponderName()
   {
      return AutoResponderReceiver;
   }


   /**
    * Method declaration
    * @return
    */
   final private UserList getUserList()
   {
      return getConfiguration().getUserList();
   }


   /**
    * Method declaration
    *
    *
    * @param receiver
    *
    * @param status
    *
    *
    * @return
    *
    */
   final private String getConfirmationMessage( String receiver, String status )
   {
      StringBuffer   sb = new StringBuffer( 512 );
      DateFormat     dateformat = new SimpleDateFormat( "E, dd MMM yyyy kk:mm:ss z", Locale.US );
      String         localDomains = getConfiguration().getLocalDomain();
      String         localDomain = null;
      int            ix = localDomains.indexOf( ';' );

      if ( ix >= 0 )
      {

         // in case we have more than one local domain specified
         // just take the first one

         localDomain = localDomains.substring( 0, ix );
      }
      else
      {
         localDomain = localDomains;
      }

      sb.append( "From: <autoresponder@" ).append( localDomain );
      sb.append( ">\nTo: " ).append( receiver );
      sb.append( "\nSubject: Request confirmation: Autoresponder status = " ).append( status );
      sb.append( "\nDate: " ).append( dateformat.format( new Date() ) );
      sb.append( "\n\nDear Java Mail Server user,\n\nThis is a confirmation message for your recent request." );
      sb.append( "\nThese settings stay active until you modify them again." );
      sb.append( "\n\nHave a nice day." );

      return sb.toString();
   }


   // checks for an existing autoresponse message
   // and uses it for creating an response

   /**
    * Method declaration
    * @param name
    * @param msg
    * @return
    * @throws Exception
    */
   final boolean autoReply( String name ) throws Exception
   {
      if ( checkForAutoreplyMessage( name ) == false )
      {
         return false;
      }

      MailMessage          message = getAutoreplyMessage( name );

      // there is a message ...
      // get Receiver of this reply ( the original sender ..)

      String               Receiver = null;     // the one to reply to
      String               Receivername = null;
      boolean              localreply = false;
      ByteArrayInputStream inputstream = new ByteArrayInputStream( message.getMessage().getBytes() );
      BufferedReader       bufferedreader = new BufferedReader( new InputStreamReader( inputstream ) );
      String               line = null;

      while ( ( line = bufferedreader.readLine() ) != null )
      {
         if ( ( line.toUpperCase().startsWith( "FROM:" ) ) || ( line.toUpperCase().startsWith( "REPLYTO:" ) ) )
         {
            try
            {
               StringTokenizer   stringtokenizer = new StringTokenizer( line, "<>" );

               stringtokenizer.nextToken();

               Receiver = stringtokenizer.nextToken();

               // get name of user that send this message to check if
               // a local reply is needed

               stringtokenizer = new StringTokenizer( Receiver, "<@>" );
               Receivername = stringtokenizer.nextToken().toLowerCase();

               String   domain = stringtokenizer.nextToken();

               if ( getConfiguration().getLocalDomain().equals( domain.toLowerCase() ) )
               {
                  localreply = true;
               }
            }
            catch ( Exception x )
            {

               // just for the case

            }

            break;
         }
      }

      inputstream.close();
      bufferedreader.close();

      if ( ( Receiver == null ) || ( Receiver.length() == 0 ) )
      {
         return false;
      }

      StringBuffer         newMsg = new StringBuffer( ( localreply ? "" : "<autoresponder@" + getConfiguration().getLocalDomain() + ">\n" ) );
      ByteArrayInputStream inputstream1 = new ByteArrayInputStream( message.getMessage().getBytes() );
      BufferedReader       bufferedreader1 = new BufferedReader( new InputStreamReader( inputstream1 ) );
      boolean              startupskipped = false;

      line = null;

      while ( ( line = bufferedreader1.readLine() ) != null )
      {
         if ( line.toUpperCase().startsWith( "MESSAGE-ID:" ) )
         {
            if ( !localreply )
            {
               newMsg.append( "<" + Receiver + ">;\n" );
            }

            startupskipped = true;
         }

         if ( line.toUpperCase().startsWith( "FROM:" ) )
         {
            line = "From: \"Autoresponder\" <autoresponder@" + getConfiguration().getLocalDomain() + ">";
            startupskipped = true;
         }
         else if ( line.toUpperCase().startsWith( "TO:" ) )
         {
            line = "To: <" + Receiver + ">";
         }
         else if ( line.toUpperCase().startsWith( "SUBJECT:" ) )
         {
            DateFormat  dateformat = DateFormat.getDateTimeInstance( DateFormat.DEFAULT, DateFormat.DEFAULT, Locale.getDefault() );

            // there should be a meaningfull subject line, but at the moment there
            // is no access to the subject of the msg to which this reply is send to.
            // StringTokenizer stringtokenizer = new StringTokenizer(line, ":");
            // stringtokenizer.nextToken();
            // line = "Subject: autoresponse: "+stringtokenizer.nextToken();

            line = "Subject: Autoresponse Message (" + dateformat.format( new Date() ) + ")";
         }

         if ( line.toUpperCase().startsWith( "DATE:" ) )
         {

            // Date: Sun, 24 Oct 1999 14:47:33 +0200

            DateFormat  dateformat = new SimpleDateFormat( "E, dd MMM yyyy kk:mm:ss z", Locale.US );

            line = "Date: " + dateformat.format( new Date() );
         }

         if ( startupskipped )
         {
            newMsg.append( line ).append( "\n" );
         }
      }

      bufferedreader1.close();
      inputstream1.close();
      getConfiguration().getMessageHandler().saveMessage( ( localreply ) ? getConfiguration().getReceiverName( Receivername ) : Sender.getOutgoingMailQueueName(), newMsg.toString() );
      ErrorReporter.getInstance().writeStatusMsg( "Autoresponse send to <" + Receiver + "> for local user: " + name );

      return ( localreply == false );     // return true ro trigger sender demon only
   }


   /**
    * Method declaration
    * @param user
    * @return
    */
   final private boolean checkForAutoreplyMessage( String user )
   {
      return ( getConfiguration().getMessageHandler().getNrOfMessages( getAutoResponderName() + user ) > 0 );
   }


   /**
    * Method declaration
    *
    *
    * @param user
    *
    *
    */
   final private void removeAutoreplyMessage( String user )
   {
      MailMessage message = getAutoreplyMessage( user );

      if ( message != null )
      {
         getConfiguration().getMessageHandler().deleteMessage( message );
      }
   }


   /**
    * Method declaration
    * @param user
    * @return
    */
   final private MailMessage getAutoreplyMessage( String user )
   {
      MailMessage[]  messages = getConfiguration().getMessageHandler().getMessages( getAutoResponderName() + user );

      if ( messages != null )
      {
         if ( getConfiguration().getMessageHandler().getNextMessage( messages[ 0 ] ) )
         {
            return messages[ 0 ];
         }
         else
         {
            return null;
         }
      }
      else
      {
         return null;
      }
   }


   // autoresponder@jps.at
   // user;pwd;on|off

   /**
    * Method declaration
    * @param filename of temp message
    * @throws Exception
    */
   final void manageAutoresponder( String filename ) throws Exception
   {
      boolean        isValidUser = false;
      String         user = null;
      String         function = null;
      boolean        localreply = false;
      String         Receivername = null;
      String         ConfirmationReceiver = null;
      BufferedReader bufferedreader = new BufferedReader( new FileReader( filename ) );
      String         line = null;

      while ( ( line = bufferedreader.readLine() ) != null )
      {
         if ( line.toUpperCase().startsWith( "FROM:" ) )
         {
            try
            {
               StringTokenizer   stringtokenizer = new StringTokenizer( line, "<>" );

               stringtokenizer.nextToken();

               // this is the sender of this email
               // he will get a confirmation about his command

               ConfirmationReceiver = stringtokenizer.nextToken();
               stringtokenizer = new StringTokenizer( ConfirmationReceiver, "<@>" );
               Receivername = stringtokenizer.nextToken().toLowerCase();

               String   domain = stringtokenizer.nextToken();

               if ( getConfiguration().getLocalDomain().equals( domain.toLowerCase() ) )
               {
                  localreply = true;
               }
            }
            catch ( NullPointerException npx )
            {

               // writeErrorMsg("Error in autoresponse remotecontrol by User: "+ user);

            }
         }
         else if ( line.toUpperCase().startsWith( "SUBJECT:" ) )
         {
            StringTokenizer   stringtokenizer = new StringTokenizer( line, " ;" );

            try
            {
               stringtokenizer.nextToken();

               user = stringtokenizer.nextToken().toLowerCase();

               String   pwd = stringtokenizer.nextToken().toLowerCase();

               function = stringtokenizer.nextToken().toLowerCase();

               if ( getUserList().getPassword( user ).equals( pwd ) )
               {
                  isValidUser = true;
               }
            }
            catch ( NullPointerException npx )
            {

               // writeErrorMsg("Error in autoresponse remotecontrol by User: "+ user);

            }

            break;
         }
      }

      if ( isValidUser )
      {
         String   status = null;

         if ( function.equals( "on" ) )
         {

            // delete old maybe existing one

            removeAutoreplyMessage( user );

            // create new autoresponse message

            getConfiguration().getMessageHandler().saveMessage( filename, getAutoResponderName() + getConfiguration().getReceiverName( Receivername ), null, null );

            // send confirmation back

            status = "ON";
         }
         else if ( function.equals( "off" ) )
         {
            removeAutoreplyMessage( user );

            status = "OFF";
         }

         if ( ConfirmationReceiver != null )
         {

            // String   confirmMsg = "From: <autoresponder@" + getConfiguration().getLocalDomain() + ">\nTo: " +
            // ConfirmationReceiver + "\nSubject: Request confirmation: Autoresponder status = " + status + "\n\nDear Java
            // Mail Server user !\n\n This is just a confirmation for your recent request.";

            getConfiguration().getMessageHandler().saveMessage( ( localreply ) ? getConfiguration().getReceiverName( Receivername ) : Sender.getOutgoingMailQueueName(), getConfirmationMessage( ConfirmationReceiver, status ) );
            ErrorReporter.getInstance().writeStatusMsg( "Autoresponse for local user: " + user + " Status changed to : " + status );
         }
      }
   }


}



/*--- formatting done in "JPS Convention" style on 08-20-2001 ---*/

