/*--- formatted by Jindent 2.1, (www.c-lab.de/~jindent) ---*/

/*
 * Copyright (c) 1999 / 2000  Johannes Plachy & Computer Mutter
 *
 * JMailSrv@jps.at
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * If you modify this file, please send us a copy.
 */

/********************************************************************************************
* CVS Info
*
* $Source: /cvsroot/jmailsrv/jmailserver/at/jps/mailserver/ErrorReporter.java,v $
*
* $Author: furykid $
* $Date: 2003/05/29 18:04:47 $
*
* modification history:
*
* $Log: ErrorReporter.java,v $
* Revision 1.4  2003/05/29 18:04:47  furykid
* rebuild 290503
*
* Revision 1.3  2002/06/27 21:54:58  furykid
* adapted to new remotepackage requirements and the logging settings
*
* Revision 1.2  2002/01/20 16:07:13  furykid
* added new header for cvs information
*

*********************************************************************************************/


package at.jps.mailserver;

/**
 * Manages the different logfiles
 * @author  Johannes Plachy
 * @version 0.99, 11/22/99
 */
final public class ErrorReporter implements CFGUpdateable
{

   /**
    * default logfilename
    */
   //private final static String   cvLogFilename = "JMailServ.log";

   /**
    * instance of this singleton
    */
   private static ErrorReporter  instance = null;

   /**
    * writer for Pop3 Messages
    */
   private LogFileWriter         ivPop3LogWriter;

   /**
    * writer for SMTP Messages
    */
   private LogFileWriter         ivSmtpLogWriter;

   /**
    * writer for Sender Messages
    */
   private LogFileWriter         ivSenderLogWriter;

   /**
    * writer for Fetcher Messages
    */
   private LogFileWriter         ivFetcherLogWriter;

   /**
    * writer for Status Messages
    */
   private LogFileWriter         ivStatusLogWriter;

   /**
    * writer for transaction Messages
    */
   private LogFileWriter         ivTXLogWriter;

   /**
    * configuration
    */
   private ConfigurationManager  ivConfigurationManager;
   private TXMsgListener         ivTXlistener = null;

   /**
    * private constructor
    */
   private ErrorReporter() {}


   /**
    * returns configuration information
    * @return configuration
    */
   final private ConfigurationManager getConfiguration()
   {
      return ivConfigurationManager;
   }


   /**
    * update configuration information
    * @return new configuration information
    */
   public void updateCFG( ConfigurationManager ConfigurationManager )
   {
      ivConfigurationManager = ConfigurationManager;

      if ( ConfigurationManager != null )
      {
         ivPop3LogWriter = null;
         ivSmtpLogWriter = null;
         ivSenderLogWriter = null;
         ivFetcherLogWriter = null;
         ivStatusLogWriter = null;
         ivTXLogWriter = null;
      }
   }


   /**
    * returns instance of the singleton reporter class
    * @return instance to this class
    */
   final public static ErrorReporter getInstance()
   {
      if ( instance == null )
      {
         instance = new ErrorReporter();
      }

      return instance;
   }


   /**
    * write to POP3 logfile
    * @param line to be written to file
    */
   final public void writePop3Msg( String s )
   {
      if ( getConfiguration().isPOP3LogFileEnabled() == false)
      {
          return;
      }

      if ( s == null )
      {
         return;
      }

      if ( ivPop3LogWriter == null )
      {
         ivPop3LogWriter = new LogFileWriter( getConfiguration().getPOP3LogFile(), "", "" );

         // write welcome msg

         ivPop3LogWriter.write( "--- POP3 Messagelog started ---" );
      }

      if ( ivPop3LogWriter != null )
      {
         ivPop3LogWriter.write( s );
      }
   }


   /**
    * write to POP3 logfile
    * @param line to be written to file
    * @param severity description
    */
   final public void writePop3Msg( String s, String severity )
   {
      if ( s == null )
      {
         return;
      }

      writePop3Msg( s + " <- " + severity );

   }


   /**
    * write to SMTP logfile
    * @param line to be written to file
    */
   final public void writeSmtpMsg( String s )
   {

        if ( getConfiguration().isSMTPLogFileEnabled() == false)
        {
           return;
        }

      if ( s == null )
      {
         return;
      }

      if ( ivSmtpLogWriter == null )
      {
         ivSmtpLogWriter = new LogFileWriter( getConfiguration().getSMTPLogFile(), "", "" );

         // write welcome msg

         ivSmtpLogWriter.write( "--- SMTP Messagelog started ---" );
      }

      if ( ivSmtpLogWriter != null )
      {
         ivSmtpLogWriter.write( s );
      }
   }


   /**
    * write to SMTP logfile
    * @param line to be written to file
    * @param severity description
    */
   final public void writeSmtpMsg( String s, String severity )
   {
      if ( s == null )
      {
         return;
      }

      writeSmtpMsg( s + " <- " + severity );

   }


   /**
    * write to Sender daemon logfile
    * @param line to be written to file
    */
   final public void writeSenderMsg( String s )
   {

       if ( getConfiguration().isSenderLogFileEnabled() == false)
       {
           return;
       }

      if ( s == null )
      {
         return;
      }

      if ( ivSenderLogWriter == null )
      {
         ivSenderLogWriter = new LogFileWriter( getConfiguration().getSenderLogFile(), "", "" );

         // write welcome msg

         ivSenderLogWriter.write( "--- Sender Messagelog started ---" );
      }

      if ( ivSenderLogWriter != null )
      {
         ivSenderLogWriter.write( s );
      }
   }


   /**
    * write to Sender daemon logfile
    * @param line to be written to file
    * @param severity description
    */
   final public void writeSenderMsg( String s, String severity )
   {
      if ( s == null )
      {
         return;
      }

      writeSenderMsg( s + " <- " + severity );

   }


   /**
    * write to Fetcher daemon logfile
    * @param line to be written to file
    */
   final public void writeFetcherMsg( String s )
   {

       if ( getConfiguration().isFetcherLogFileEnabled() == false)
       {
           return;
       }

      if ( s == null )
      {
         return;
      }

      if ( ivFetcherLogWriter == null )
      {
         ivFetcherLogWriter = new LogFileWriter( getConfiguration().getFetcherLogFile(), "", "" );

         // write welcome msg

         ivFetcherLogWriter.write( "--- Fetcher Messagelog started ---" );
      }

      if ( ivFetcherLogWriter != null )
      {
         ivFetcherLogWriter.write( s );
      }
   }


   /**
    * write to Fetcher daemon logfile
    * @param line to be written to file
    * @param severity description
    */
   final public void writeFetcherMsg( String s, String severity )
   {
      if ( s == null )
      {
         return;
      }

      writeFetcherMsg( s + " <- " + severity );

   }


   /**
    * write to Status logfile
    * @param line to be written to file
    */
   final public void writeStatusMsg( String s )
   {
       if ( getConfiguration().isStatusLogFileEnabled() == false)
       {
           return;
       }


      if ( s == null )
      {
         return;
      }

      if ( ivStatusLogWriter == null )
      {
         ivStatusLogWriter = new LogFileWriter( getConfiguration().getStatusLogFile(), "", "" );
      }

      if ( ivStatusLogWriter != null )
      {
         ivStatusLogWriter.write( s );
      }
   }


   /**
    * write to Status logfile
    * @param line to be written to file
    * @param severity description
    */
   final public void writeStatusMsg( String s, String severity )
   {
      if ( s == null )
      {
         return;
      }

      writeStatusMsg( s + " <- " + severity );

   }


   /**
    * write to transaction logfile
    * @param line to be written to file
    */
   final public void writeTXMsg( String s )
   {
       if ( getConfiguration().isTransactionLogFileEnabled() == false)
       {
           return;
       }

      if ( s == null )
      {
         return;
      }

      if ( ivTXLogWriter == null )
      {
         ivTXLogWriter = new LogFileWriter( getConfiguration().getTransactionLogFile(), "", "" );
      }

      if ( ivTXLogWriter != null )
      {
         String   line = ivTXLogWriter.getDateString() + s;

         ivTXLogWriter.write( line );

         if ( ivTXlistener != null )
         {
            ivTXlistener.lineAdded( line + "\n" );
         }
      }
   }


   /**
    * Method declaration
    * @param listener
    */
   final public void setTXMsgListener( TXMsgListener listener )
   {
      ivTXlistener = listener;
   }


   /**
    * Method declaration
    */
   final public void removeTXMsgListener()
   {
      ivTXlistener = null;
   }


}



/*--- formatting done in "JPS Convention" style on 08-20-2001 ---*/

