/*--- formatted by Jindent 2.1, (www.c-lab.de/~jindent) ---*/

/*
 * Copyright (c) 1999 / 2000  Johannes Plachy & Computer Mutter
 *
 * JMailSrv@jps.at
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * If you modify this file, please send us a copy.
 */

/********************************************************************************************
* CVS Info
*
* $Source: /cvsroot/jmailsrv/jmailserver/at/jps/mailserver/Event.java,v $
*
* $Author: furykid $
* $Date: 2002/01/20 16:07:13 $
*
* modification history:
*
* $Log: Event.java,v $
* Revision 1.2  2002/01/20 16:07:13  furykid
* added new header for cvs information
*

*********************************************************************************************/


package at.jps.mailserver;

/**
 * This class simulates an EventSemaphore where a thread can be waiting for an event and
 * can be notified by another thread to continue execution
 * @author  Johannes Plachy
 * @version 0.99, 11/22/99
 */
final class Event
{
   final static private int   WAITTIMEOUT = 20;    // 20 min

   /**
    * constructor initializes the internal state to be not signalled
    */
   public Event()
   {
      signaled = false;
   }


   /**
    * Method declaration
    *
    *
    * @return
    *
    */
   final public static int getDefaultWaitTimeout()
   {
      return WAITTIMEOUT * 60 * 1000;
   }


   /**
    * called by a thread wanting to wait for a specific external statechange.
    * It waits for another thread calling the trigger method to wake him up again
    */
   final public synchronized void waitForEvent()
   {
      while ( !signaled )
      {
         try
         {
            wait( getDefaultWaitTimeout() );    // should be changed !!!!
         }
         catch ( InterruptedException ex ) {}
      }

      signaled = false;
   }


   /**
    * trigger notifies all waiting threads to continue execution
    */
   final public synchronized void trigger()
   {
      signaled = true;

      notifyAll();
   }


   /**
    * state of the eventsemaphore
    */
   private boolean   signaled;
}



/*--- formatting done in "JPS Convention" style on 08-20-2001 ---*/

