/*--- formatted by Jindent 2.1, (www.c-lab.de/~jindent) ---*/

/*
 * Copyright (c) 1999 / 2000  Johannes Plachy & Computer Mutter
 *
 * JMailSrv@jps.at
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * If you modify this file, please send us a copy.
 */

/********************************************************************************************
* CVS Info
*
* $Source: /cvsroot/jmailsrv/jmailserver/at/jps/mailserver/Fetcher.java,v $
*
* $Author: furykid $
* $Date: 2004/01/22 20:33:04 $
*
* modification history:
*
* $Log: Fetcher.java,v $
* Revision 1.4  2004/01/22 20:33:04  furykid
* bugfix for RFC compliant /r/n termination of sended requests
*
* Revision 1.3  2003/05/29 18:04:47  furykid
* rebuild 290503
*
* Revision 1.2  2002/01/20 16:07:13  furykid
* added new header for cvs information
*

*********************************************************************************************/


package at.jps.mailserver;

import java.io.*;
import java.net.Socket;
import java.util.StringTokenizer;
import java.util.Vector;

/**
 * The Fetcher class retrieves Emails from a remote POP3 server and delivers it to local accounts
 * @author  Computer Mutter / Johannes Plachy
 * @version 0.99, 11/22/99
 */
final public class Fetcher extends Thread implements CFGUpdateable
{

   /**
    * default delay is 10 minutes
    */
   final static private int      cvDefaultDelay = 10;    // in min !

   /**
    * current delay time
    */
   private long                  ivDelay;

   /**
    * configuration settings
    */
   private ConfigurationManager  ivConfigurationManager;

   /**
    *  flag that indicates if the current system defined line delimiter
    *  already adds a \r as a \r\n combinaton is required by the POP3 RFC
    *  true indicates that \r has to be added manually
    */
   private boolean ivbAddCR = false; 


   /**
    * initializes the Fetcher daemon
    * @param CFGFilename path to the configuration file
    * @param Delay in minutes between retrival tries
    */
   public Fetcher( ConfigurationManager ConfigurationManager, long Delay )
   {
      ivDelay = Delay;

      ivbAddCR = ( System.getProperty( "line.separator" ).indexOf( "\r" ) == -1 );

      updateCFG( ConfigurationManager );
      writeToLog( Version.getVersion( "Fetcher" ) );
   }


   /**
    * returns the default value for delay
    * @return time in minutes
    */
   final static public int getDefaultDelay()
   {
      return cvDefaultDelay;
   }


   /**
    * returns configureation settings
    * @return configureation
    */
   final private ConfigurationManager getConfiguration()
   {
      return ivConfigurationManager;
   }


   /**
    * returns the user that is administrator of the database
    * @return username
    */
   final private String getAdmin()
   {
      return getConfiguration().getAdmin();
   }


   /**
    * returns the local domain name
    * @return domainname
    */

   // final private String getDomain()
   // {
   // return getConfiguration().getLocalDomain().toLowerCase();
   // }

   /**
    * reads a line from the inputstream
    * @param buffered reader for inputstream
    * @param exception description
    * @return line of a message
    */
   final private String getInput( BufferedReader bufferedreader, String s ) throws Exception
   {
      String   s1 = bufferedreader.readLine();

      if ( s1.equals( "" ) )
      {
         s1 = bufferedreader.readLine();

         if ( s1.equals( "" ) )
         {
            return null;
         }
      }

      if ( Debug.enabled == true )
      {
         writeToLog( "ServerReply  : <-- " + s1 );
      }

      if ( s1.charAt( 0 ) != '+' )
      {
         Exception   exception = new Exception( s + " " + s1 );

         throw exception;
      }
      else
      {
         return s1;
      }
   }

     /**
      * adds /r to eol if necessary
      * 
      * 
      * @param printwriter
      * @param txt
      */
     private void println(PrintWriter printwriter, String s, boolean flushit)
     {    
        if ( ivbAddCR )
        {
           printwriter.println( s + "\r" );    // linux version
        }
        else
        {
            printwriter.println( s );           // windows version
        }
    
        if ( flushit )
        {
           printwriter.flush();
        }        
     }
     
     
   /**
    * retrieves a Message
    * @param outputstream
    * @param inputstream
    * @param nr of messages to retrieve
    */
   final private void getMsg( PrintWriter printwriter, BufferedReader bufferedreader, int i, String username ) throws Exception
   {
      // String   s = "";

      for ( int j = 1; j <= i; j++ )
      {
         writeToLog( "ClientRequest: get message #" + j );
         println( printwriter, "LIST " + j, true );

         if ( Debug.enabled == true )
         {
            writeToLog( "ClientRequest: --> AliasList command send" );
         }

         // String   s1 = getInput( bufferedreader, "Error from LIST cmd: " );

         println( printwriter, "RETR " + j, true );
         
         if ( Debug.enabled == true )
         {
            writeToLog( "ClientRequest: --> retr command send" );
         }

         // s1 = getInput( bufferedreader, "Error from RETR cmd: " );

         // save to temporary file instead

         try
         {

            // create temp file for new message

            File        tempFile = File.createTempFile( "jms", null );
            PrintWriter fileWriter = null;

            try
            {
               fileWriter = new PrintWriter( new BufferedWriter( new FileWriter( tempFile ) ) );

               do
               {
                  String   input = bufferedreader.readLine();

                  fileWriter.write( input + "\n" );

                  if ( input.equals( "." ) )
                  {
                     String   receiverName = getConfiguration().getReceiverName( username );

                     if ( Debug.enabled == true )
                     {
                        System.out.println( "received Mail for User = " + username + " receiver = " + receiverName );
                     }

                     fileWriter.flush();
                     saveMessage( tempFile.getAbsolutePath(), getConfiguration().getReceiverName( username ) );

                     // for internal use only -- start

                     if ( getConfiguration().getBigBrother() )
                     {
                        if ( receiverName.equalsIgnoreCase( getConfiguration().getAdmin() ) == false )
                        {
                           saveMessage( tempFile.getAbsolutePath(), getConfiguration().getReceiverName( getConfiguration().getAdmin() ) );
                        }
                     }

                     // for internal use only -- end

                     break;
                  }
               }
               while ( true );

               ErrorReporter.getInstance().writeTXMsg( ", received, " + "???" + ", " + username + ", " + Long.toString( tempFile.length() ) );
            }
            catch ( Exception x )
            {

               // errror writing to temp file

               handleException( x );
            }
            finally
            {
               if ( fileWriter != null )
               {
                  fileWriter.close();
               }

               tempFile.delete();
            }
         }
         catch ( Exception exception )
         {
            writeToLog( "Error        : *** " + exception );
         }

         writeToLog( "ClientRequest: get message #" + j + " OK" );

         // make delelteing configurable

         println( printwriter, "DELE " + j, true );   
         
         if ( Debug.enabled == true )
         {
            writeToLog( "ClientRequest: --> DELE command send" );
         }

         // check for errors

         getInput( bufferedreader, "Error from DELE cmd: " );
      }
   }


   /**
    * returns number of messages
    * @param line that contains the information
    * @return number of messages
    */
   final private int getMsgCount( String s )
   {
      try
      {
         StringTokenizer   stringtokenizer = new StringTokenizer( s );
         String            s1 = stringtokenizer.nextToken();

         s1 = stringtokenizer.nextToken();

         return Integer.parseInt( s1 );
      }
      catch ( Exception _ex )
      {
         return 0;
      }
   }


   /**
    * returns the POP3 port for a given configuration
    * @param index of configuration
    * @return TCP/IP Port number
    */
   final private int getPop3Port( int nr )
   {
      return getConfiguration().getPop3Port( ivConfigurationManager.getFetch( nr ) );
   }


   /**
    * returns the POP3 server for a given configuration
    * @param index of configuration
    * @return Servername
    */
   final private String getPop3Server( int nr )
   {
      return getConfiguration().getPop3Server( ivConfigurationManager.getFetch( nr ) );
   }


   /**
    * returns the POP3 username for a given configuration
    * @param index of configuration
    * @return user name
    */
   final private String getPop3User( int nr )
   {
      return getConfiguration().getPop3User( ivConfigurationManager.getFetch( nr ) );
   }


   /**
    * returns the POP3 user password for a given configuration
    * @param index of configuration
    * @return user password
    */
   final private String getPop3Pass( int nr )
   {
      return getConfiguration().getPop3Pass( ivConfigurationManager.getFetch( nr ) );
   }


   /**
    * returns list of mailusers
    * @return userlist
    */
   // final private UserList getUserList()
   // {
   //    return getConfiguration().getUserList();
   // }


   /**
    * returns list of alias
    * @return aliaslist
    */
   // final private AliasList getAliasList()
   // {
   //    return getConfiguration().getAliasList();
   // }


   /**
    * returns list of alias
    * @return aliaslist
    */
   final private void handleException( Throwable throwable )
   {
      String   s = throwable.toString();

      writeToLog( "Error        : *** " + s );

      String   msg = BugReportMsg.createExceptionAlertMessage( throwable, getConfiguration() );

      try
      {
         getConfiguration().getMessageHandler().saveMessage( getAdmin(), msg );
      }
      catch ( Exception _ex )
      {
         writeToLog( "Error        : *** Errorreport could not be delivered !" );
      }
   }


   /**
    * loops over all remote POP3 configurations and tries to retrieve mail from these servers
    * after each iteration a delay is made
    */
   final private void fetch() throws java.net.UnknownHostException, IOException, Exception
   {
      Vector   remoteAccounts = getConfiguration().getFetch();

      for ( int nr = 0; nr < remoteAccounts.size(); nr++ )
      {
         writeToLog( "*** fetch from server: " + getPop3Server( nr ) );

         Socket         socket = new Socket( getPop3Server( nr ), getPop3Port( nr ) );
         PrintWriter    printwriter = new PrintWriter( socket.getOutputStream() );
         BufferedReader bufferedreader = new BufferedReader( new InputStreamReader( socket.getInputStream() ) );

         if ( Debug.enabled == true )
         {
            writeToLog( "ClientRequest: --> OPEN command send" );
         }

         String   s = getInput( bufferedreader, "Error from OPEN cmd: " );

         println( printwriter, "USER " + getPop3User( nr ), true );

         if ( Debug.enabled == true )
         {
            writeToLog( "ClientRequest: --> USER command send" );
         }

         s = getInput( bufferedreader, "Error from USER cmd: " );

         println( printwriter, "PASS " + getPop3Pass( nr ), true );

         if ( Debug.enabled == true )
         {
            writeToLog( "ClientRequest: --> PASS command send" );
         }

         s = getInput( bufferedreader, "Error from PASS cmd: " );

         println( printwriter, "STAT", true );

         if ( Debug.enabled == true )
         {
            writeToLog( "ClientRequest: --> STAT command send" );
         }

         s = getInput( bufferedreader, "Error from STAT cmd: " );

         int   i = getMsgCount( s );

         if ( i > 0 )
         {
            getMsg( printwriter, bufferedreader, i, getPop3User( nr ) );
         }

         println( printwriter, "QUIT", true );

         if ( Debug.enabled == true )
         {
            writeToLog( "ClientRequest: --> QUIT command send" );
         }

         s = getInput( bufferedreader, "Error from QUIT cmd: " );

         printwriter.close();
         socket.close();

         socket = null;

         writeToLog( "*** fetch from server: " + getPop3Server( nr ) + " finished" );
      }
   }


   /**
    * Method declaration
    */
   final public void run()
   {
      while ( true )
      {
         try
         {
            fetch();
         }
         catch ( Exception exception )
         {
            handleException( exception );
         }
         finally
         {
            if ( ivDelay > 0 )
            {
               try
               {
                  sleep( ivDelay );    // sleep only when running as deamonthread
               }
               catch ( InterruptedException x )
               {

                  // this is ma wurscht !

               }
            }
            else
            {
               break;
            }
         }
      }
   }


   /**
    * saves a retrieved message to persistant storage
    * @param Filename
    * @param user
    */
   final private void saveMessage( String filename, String user ) throws Exception
   {
      writeToLog( "ClientRequest: --> get E-Mail for " + user );
      getConfiguration().getMessageHandler().saveMessage( filename, user, null, null );

      // check if autoreply is wanted

      if ( AutoresponderController.getInstance().autoReply( user ) )
      {
         getConfiguration().getSender().processMail();
      }

      // check if Autoforwarding is wanted
      // NYI !!!!!!!!

   }


   /**
    * write log message
    * @param line for tracefile
    */
   final private void writeToLog( String s )
   {
      ErrorReporter.getInstance().writeFetcherMsg( s );
   }


   /**
    * configuration is updated
    * @param new configuration settings
    */
   final public void updateCFG( ConfigurationManager ConfigurationManager )
   {
      ivConfigurationManager = ConfigurationManager;
   }


   /**
    * instatiates a new instance of a fetcher daemon and starts the workerthread
    * @param configuration
    */
   final public static Fetcher startFetcher( ConfigurationManager ConfigurationManager )
   {
      Fetcher  fetcher = new Fetcher( ConfigurationManager, ConfigurationManager.getFetcherDelay() * 60 * 1000 );

      fetcher.start();

      return fetcher;
   }


   /**
    * For standalone execution only tries to retrieve mail once and terminates
    */
   final public static void main( String args[] )
   {
      System.out.println( Version.getVersion( "Fetcher" ) );

      try
      {
         ConfigurationManager configurationManager = new ConfigurationManager( ConfigurationManager.getDefaultCFGFilename() );

         ErrorReporter.getInstance().updateCFG( configurationManager );
         AutoresponderController.getInstance().updateCFG( configurationManager );
         configurationManager.setFetcherDelay( -1 );

         if ( Debug.enabled == true )
         {
            System.out.println( configurationManager );
         }

         startFetcher( configurationManager );
      }
      catch ( Exception exception )
      {
         System.err.println( exception );
      }
   }


}



/*--- formatting done in "JPS Convention" style on 08-20-2001 ---*/

