/*--- formatted by Jindent 2.1, (www.c-lab.de/~jindent) ---*/

/*
 * Copyright (c) 1999 / 2000  Johannes Plachy & Computer Mutter
 *
 * JMailSrv@jps.at
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * If you modify this file, please send us a copy.
 */

/********************************************************************************************
* CVS Info
*
* $Source: /cvsroot/jmailsrv/jmailserver/at/jps/mailserver/LogFileWriter.java,v $
*
* $Author: furykid $
* $Date: 2002/01/20 16:07:13 $
*
* modification history:
*
* $Log: LogFileWriter.java,v $
* Revision 1.2  2002/01/20 16:07:13  furykid
* added new header for cvs information
*

*********************************************************************************************/


package at.jps.mailserver;

import java.io.FileWriter;
import java.text.DateFormat;
import java.util.Date;
import java.util.Locale;

/**
 * This class just writes a string to the logfile and makes it easy to format a line
 * @author  Johannes Plachy
 * @version 0.99, 11/22/99
 */
final public class LogFileWriter
{

   /**
    * Logfile name
    */
   private String             ivFileName;

   /**
    * String to be prepended to each line passed to the logging method
    */
   private String             ivStartString;

   /**
    * String to be appended to each line passed to the logging method
    */
   private String             ivEndString;
   private static DateFormat  cvDateformat = DateFormat.getDateTimeInstance( DateFormat.DEFAULT, DateFormat.DEFAULT, Locale.getDefault() );

   /**
    * if the counter is greater than MAX_ERR logging will be switched to STDERR
    */
   private int                errCounter = 0;
   private final static int   MAX_ERR = 3;

   /**
    * initializes the writer with
    * @param file to write output to
    * @param prepend-string
    * @param append-string
    */
   public LogFileWriter( String LogFilename, String StartString, String EndString )
   {
      setFileName( LogFilename );
      setStartString( StartString );
      setEndString( EndString );
   }


   /**
    * returns appendd-string
    * @return append-string
    */
   final private String getEndString()
   {
      return ivEndString;
   }


   /**
    * returns filename of logfile
    * @return filename
    */
   final public String getFileName()
   {
      return ivFileName;
   }


   /**
    * returns prepend-string
    * @return prepend-string
    */
   final private String getStartString()
   {
      return ivStartString;
   }


   /**
    * set append-string
    * @param append-string
    */
   final public void setEndString( String s )
   {
      ivEndString = s;
   }


   /**
    * set filename for logfile
    * @param filename
    */
   final public void setFileName( String s )
   {
      ivFileName = s;
   }


   /**
    * set prepend-string
    * @param prepend-string
    */
   final public void setStartString( String s )
   {
      ivStartString = s;
   }


   /**
    * return Date represented as String
    * @param string current date
    */
   String getDateString()
   {
      return cvDateformat.format( new Date() );
   }


   /**
    * write string to logfile
    * @param string to be logged to file
    */
   final public void write( String s )
   {
      try
      {
         StringBuffer   sb = new StringBuffer( getStartString() );

         sb.append( s ).append( " (" ).append( getDateString() ).append( ")" ).append( getEndString() ).append( "\n" );

         if ( ( errCounter >= MAX_ERR ) || ( Debug.enabled ) )
         {
            System.err.print( sb.toString() );
         }

         if ( errCounter < MAX_ERR )
         {
            FileWriter  writer = new FileWriter( getFileName(), true );

            // writer.println(sb.toString());

            writer.write( sb.toString() );
            writer.flush();
            writer.close();
         }
      }
      catch ( Exception exception )
      {
         exception.printStackTrace();

         errCounter++;
      }
   }


}



/*--- formatting done in "JPS Convention" style on 08-20-2001 ---*/

