/*--- formatted by Jindent 2.1, (www.c-lab.de/~jindent) ---*/

/*
 * Copyright (c) 1999 / 2000  Johannes Plachy & Computer Mutter
 *
 * JMailSrv@jps.at
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * If you modify this file, please send us a copy.
 */

/********************************************************************************************
* CVS Info
*
* $Source: /cvsroot/jmailsrv/jmailserver/at/jps/mailserver/MailStatistic.java,v $
*
* $Author: furykid $
* $Date: 2002/01/20 16:07:13 $
*
* modification history:
*
* $Log: MailStatistic.java,v $
* Revision 1.2  2002/01/20 16:07:13  furykid
* added new header for cvs information
*

*********************************************************************************************/


package at.jps.mailserver;

import java.util.*;

/**
 * provides statistical information about the mailtraffic
 * @author  Johannes Plachy
 * @version 0.99, 11/22/99
 */
public final class MailStatistic implements MailServerAdmin
{

   /**
    * time the server has been started
    */
   private static Date           cvStartTime = new Date();

   /**
    * number of incoming mails
    */
   private static long           cvIncomingMail = 0;

   /**
    * number of outgoing mails
    */
   private static long           cvOutgoingMail = 0;

   /**
    * size of incoming mails
    */
   private static long           cvIncomingSize = 0;

   /**
    * size of outgoing mails
    */
   private static long           cvOutgoingSize = 0;

   /**
    * instance of this singleton
    */
   private static MailStatistic  cvInstance = null;

   /**
    * instance of this singleton
    */
   private static Vector         ivListeners = null;

   /**
    * only for internal use
    */
   private MailStatistic()
   {
      ivListeners = new Vector();
   }


   /**
    * try if there is an instance of this class
    * @return true if in use
    */
   final static boolean isInitialized()
   {
      return ( cvInstance != null );
   }


   /**
    * get instance of this class
    * @return instance
    */
   final static public MailStatistic getInstance()
   {
      if ( cvInstance == null )
      {
         cvInstance = new MailStatistic();
      }

      return cvInstance;
   }


   /**
    * get the date and time the server was started
    * @return start date
    */
   final public Date getStartTime()
   {
      return cvStartTime;
   }


   /**
    * get number of incoming mails
    * @return number of mails
    */
   final public long getNrOfIncomingMail()
   {
      return cvIncomingMail;
   }


   /**
    * get size of incoming mails
    * @return size of mails
    */
   final public long getSizeOfIncomingMail()
   {
      return cvIncomingSize;
   }


   /**
    * get number of outgoing mails
    * @return number of mails
    */
   final public long getNrOfOutgoingMail()
   {
      return cvOutgoingMail;
   }


   /**
    * get size of outgoing mails
    * @return size of mails
    */
   final public long getSizeOfOutgoingMail()
   {
      return cvOutgoingSize;
   }


   /**
    * set number of incoming mails
    * @param number of mails
    */
   final void setNrOfIncomingMail( long IncomingMail )
   {
      cvIncomingMail = IncomingMail;

      incomingMailAmountChanged( cvIncomingMail );
   }


   /**
    * set size of incoming mails
    * @param size of mails
    */
   final void setSizeOfIncomingMail( long IncomingSize )
   {
      cvIncomingSize = IncomingSize;

      incomingMailSizeChanged( cvIncomingSize );
   }


   /**
    * set number of outgoing mails
    * @param number of mails
    */
   final void setNrOfOutgoingMail( long OutgoingMail )
   {
      cvOutgoingMail = OutgoingMail;

      outgoingMailAmountChanged( OutgoingMail );
   }


   /**
    * set size of outgoing mails
    * @param size of mails
    */
   final void setSizeOfOutgoingMail( long OutgoingSize )
   {
      cvOutgoingSize = OutgoingSize;

      outgoingMailSizeChanged( OutgoingSize );
   }


   /**
    * Method declaration
    * @return
    */
   public Settings getSettings()
   {
      return MailServer.getInstance().getConfiguration().getSettings();
   }


   /**
    * Method declaration
    * @param settings
    */
   public void setSettings( Settings settings )
   {
      MailServer.getInstance().getConfiguration().setSettings( settings, true );
   }


   /**
    * Method declaration
    * @param User
    * @return
    */
   public boolean isAutoResponseEnabled( String User )
   {
      return true;
   }


   /**
    * Method declaration
    * @return
    */
   public ConfigurationManager getConfiguration()
   {
      return MailServer.getInstance().getConfiguration();
   }


   /**
    * Registers a Listener
    */
   public synchronized void addListener( MailStatisticsListener listener )
   {
      ivListeners.addElement( listener );
   }


   /**
    * Method declaration
    * @param listener
    */
   public synchronized void removeListener( MailStatisticsListener listener )
   {
      ivListeners.removeElement( listener );
   }


   /**
    * Returns a copy of the listeners.
    */
   private synchronized Vector cloneListeners()
   {
      return ( Vector )ivListeners.clone();
   }


   /**
    * Method declaration
    * @param newSize
    */
   private void outgoingMailSizeChanged( long newSize )
   {
      Enumeration e = cloneListeners().elements();

      while ( e.hasMoreElements() )
      {
         ( ( MailStatisticsListener )e.nextElement() ).outgoingMailSizeChanged( newSize );
      }
   }


   /**
    * Method declaration
    * @param newSize
    */
   private void incomingMailSizeChanged( long newSize )
   {
      Enumeration e = cloneListeners().elements();

      while ( e.hasMoreElements() )
      {
         ( ( MailStatisticsListener )e.nextElement() ).incomingMailSizeChanged( newSize );
      }
   }


   /**
    * Method declaration
    * @param newNumber
    */
   private void outgoingMailAmountChanged( long newNumber )
   {
      Enumeration e = cloneListeners().elements();

      while ( e.hasMoreElements() )
      {
         ( ( MailStatisticsListener )e.nextElement() ).outgoingMailAmountChanged( newNumber );
      }
   }


   /**
    * Method declaration
    * @param newNumber
    */
   private void incomingMailAmountChanged( long newNumber )
   {
      Enumeration e = cloneListeners().elements();

      while ( e.hasMoreElements() )
      {
         ( ( MailStatisticsListener )e.nextElement() ).incomingMailAmountChanged( newNumber );
      }
   }


}



/*--- formatting done in "JPS Convention" style on 08-20-2001 ---*/

