/*--- formatted by Jindent 2.1, (www.c-lab.de/~jindent) ---*/

/*
 * Copyright (c) 1999 / 2000  Johannes Plachy & Computer Mutter
 *
 * JMailSrv@jps.at
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * If you modify this file, please send us a copy.
 */

/********************************************************************************************
* CVS Info
*
* $Source: /cvsroot/jmailsrv/jmailserver/at/jps/mailserver/MessageHandlerFactory.java,v $
*
* $Author: furykid $
* $Date: 2003/04/01 19:08:12 $
*
* modification history:
*
* $Log: MessageHandlerFactory.java,v $
* Revision 1.4  2003/04/01 19:08:12  furykid
* cleanup
*
* Revision 1.3  2002/01/20 16:07:13  furykid
* added new header for cvs information
*

*********************************************************************************************/


package at.jps.mailserver;


/**
 * Factory for Messagehandlers which manage messages. Either a JDBC based Handler is returned
 * or a file based Handler, depending on the configuration setting
 * @author  Johannes Plachy
 * @version 0.99, 11/22/99
 */
class MessageHandlerFactory
{

   /**
    * instance of the singleton
    */
   private static MessageHandlerFactory   cvInstance = null;

   /**
    * specific Messagehandler
    */
   private static MessageHandler          cvMessageHandler = null;

   /**
    * private constructor
    */
   private MessageHandlerFactory() {}


   /**
    * returns instance of the singleton
    * @return instance
    */
   final public static MessageHandlerFactory getInstance()
   {
      if ( cvInstance == null )
      {
         cvInstance = new MessageHandlerFactory();
      }

      return cvInstance;
   }


   /**
    * based on the passed configuration a MessageHandler is instantiated either filesystem or JDBC based
    * @param configuration
    * @return MessageHandler
    */
   final public MessageHandler getMessageHandler( ConfigurationManager configurationManager )
   {

      // first try to instantiate the JDBC version
      // afterwarts use the std method

      if ( cvMessageHandler == null )
      {
         if ( configurationManager.isJDBCendabled() )
         {
            cvMessageHandler = new MessageHandlerJDBC( configurationManager );

            cvMessageHandler.initialize();

            if ( cvMessageHandler.isConnected() == false )
            {
               cvMessageHandler = null;
            }
         }

         if ( cvMessageHandler == null )
         {
            cvMessageHandler = new MessageHandlerFile( configurationManager );
         }
      }

      return cvMessageHandler;
   }


}



/*--- formatting done in "JPS Convention" style on 08-20-2001 ---*/

