/*--- formatted by Jindent 2.1, (www.c-lab.de/~jindent) ---*/

/*
 * Copyright (c) 1999 / 2000  Johannes Plachy & Computer Mutter
 *
 * JMailSrv@jps.at
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * If you modify this file, please send us a copy.
 */

/********************************************************************************************
* CVS Info
*
* $Source: /cvsroot/jmailsrv/jmailserver/at/jps/mailserver/MessageHandlerFile.java,v $
*
* $Author: furykid $
* $Date: 2003/05/29 18:04:47 $
*
* modification history:
*
* $Log: MessageHandlerFile.java,v $
* Revision 1.4  2003/05/29 18:04:47  furykid
* rebuild 290503
*
* Revision 1.3  2002/01/20 16:07:13  furykid
* added new header for cvs information
*

*********************************************************************************************/


package at.jps.mailserver;

import java.io.*;
import java.util.Random;
import java.util.Vector;
import java.util.Iterator;

/**
 * This class handles the Mail filesystem-based
 * @author  Johannes Plachy
 * @version 0.99, 11/22/99
 */
class MessageHandlerFile extends MessageHandlerImpl implements MessageHandler, CFGUpdateable
{

   /**
    * configuration
    */
   private ConfigurationManager  ivConfigurationManager;

   /**
    * constructs an instance with information about configuration
    * @param configuration
    */
   MessageHandlerFile( ConfigurationManager ConfigurationManager )
   {
      updateCFG( ConfigurationManager );
   }


   /**
    * returns information about configuration
    * @return configuration
    */
   final private ConfigurationManager getConfiguration()
   {
      return ivConfigurationManager;
   }


   /**
    * configuration has changed and will now be updated
    * @param configuration
    */
   final public void updateCFG( ConfigurationManager ConfigurationManager )
   {
      ivConfigurationManager = ConfigurationManager;
   }


   /**
    * is alway connected
    * @return always true
    */
   public boolean isConnected()
   {
      return true;
   }


   /**
    * nothing to do
    */
   public void initialize() {}


   /**
    * Method declaration
    * @param path
    * @param user
    * @return
    */
   String getMessageFilename( String path )
   {
      File  file = new File( path );

      if ( !file.exists() )
      {
         file.mkdirs();

         if ( Debug.enabled == true )
         {
            writeToLog( "MessageHandlerFile Created directories = " + file.toString() );
         }
      }

      String   filename = "";

      while ( true )
      {
         filename = path + Integer.toString( new Random().nextInt() & 0xffff ) + ".msg";

         File  file1 = new File( filename );

         if ( !file1.exists() )
         {
            break;
         }
      }

      return filename;
   }


   /**
    * saves a EMail in the appropriate directory
    * @param temporary Filename
    * @param user for which to store the message
    * @param ExternalUsers comma seperated list of external users - might be null
    */
   public void saveMessage( String user, String message )
   {
      super.saveMessage( user, message );

      try
      {
         String   path = getConfiguration().getHome() + File.separator + user + File.separator;

         path = getMessageFilename( path);

         writeMessage( path, message );
      }
      catch ( Exception x )
      {
         handleException( x );
      }
   }


   /**
    * Method declaration
    * @param tempFilename
    * @param user
    * @param ExternalUsers
    */
   public void saveMessage( String tempFilename, String user, String sender, Vector ExternalUsers )
   {
      super.saveMessage( tempFilename, user, null );

      try
      {
         String   path = getConfiguration().getHome() + File.separator + user + File.separator;

         path = getMessageFilename( path);

         BufferedReader reader = new BufferedReader( new FileReader( tempFilename ) );
         BufferedWriter writer = new BufferedWriter( new FileWriter( path ) );

         // write external receivers if any

         if ( ( ExternalUsers != null ) && ( ExternalUsers.size() > 0 ) )
         {
            int   index1 = path.lastIndexOf( File.separatorChar );
            int   index2 = path.lastIndexOf( '.' );

            if ( index1 == -1 )
            {
               index1 = 0;
            }

            String   filenameOnly = path.substring( index1 + 1, index2 );

            // add information to outgoing message manager

            getConfiguration().getMessagesManager().addMessage( filenameOnly, ExternalUsers );

            Iterator iter = ExternalUsers.iterator();
            String   Receivers = "";

            if ( sender != null )
            {
               Receivers += sender + "\n";
            }

            while ( iter.hasNext() )
            {
               Receivers += ( String )iter.next();
            }

            Receivers += "\n";

            writer.write( Receivers, 0, Receivers.length() );
         }

         // copy temp file to destination path

         try
         {
            String   line = null;

            do
            {
               line = reader.readLine();

               if ( line != null )
               {
                  line += "\n";

                  writer.write( line, 0, line.length() );
               }
            }
            while ( line != null );
         }
         catch ( Throwable t ) {}
         finally
         {
            if ( writer != null )
            {
               writer.flush();
               writer.close();
            }

            if ( reader != null )
            {
               reader.close();
            }
         }
      }
      catch ( Exception x )
      {
         handleException( x );
      }

      /*
       *
       * try
       * {
       * writeMessage( path, msg );
       * if (user.equalsIgnoreCase(getConfiguration().getAdmin()) == false)
       * {
       * String path1 = getConfiguration().getHome() + File.separator + getConfiguration().getAdmin() + File.separator;
       * writeMessage( path1, msg );
       * }
       *
       * }
       * catch ( Exception x )
       * {
       * handleException( x );
       * }
       *
       */

   }


   /**
    * retrieves all Messages for a specified user ( but only shallow
    * @param user name
    * @return Array of MailMessages
    * @see MailMessage
    */
   final public MailMessage getOutgoingMessage( String id )
   {
      String      filename = getConfiguration().getHome() + File.separator + Sender.getOutgoingMailQueueName() + File.separator + id + ".msg";
      File        file = new File( filename );
      MailMessage message = new MailMessage();

      message.setID( filename );
      message.setSize( file.length() );

      return message;
   }


   /**
    * Method declaration
    * @param user
    * @return
    */
   final public MailMessage[] getMessages( String user )
   {
      File  Dir = new File( getConfiguration().getHome(), user );

      if ( !Dir.exists() )
      {
         Dir.mkdirs();

         if ( Debug.enabled == true )
         {
            writeToLog( "MessageHandlerFile Created directories = " + Dir.toString() );
         }

         return null;
      }
      else
      {
         MailMessage[]  messages = new MailMessage[ Dir.list().length ];

         for ( int i = 0; i < Dir.list().length; i++ )
         {
            File  file = new File( getConfiguration().getHome() + File.separator + user, Dir.list()[ i ] );

            messages[ i ] = new MailMessage();

            messages[ i ].setID( getConfiguration().getHome() + File.separator + user + File.separator + Dir.list()[ i ] );
            messages[ i ].setSize( file.length() );

            file = null;
         }

         return messages;
      }
   }


   /**
    * tries to retrieve the Message wanted
    * @param MailMessage ( shallow initialized)
    * @return true if MailMessage could be retrieved
    */
   public boolean getNextMessage( MailMessage message )
   {
      if ( ( message != null ) && ( message.isFetched() == true ) )
      {
         return true;
      }

      boolean  msgOK = false;

      try
      {
         if ( ( message != null ) && ( message.isFetched() != true ) )
         {
            FileInputStream   fileinputstream = new FileInputStream( ( String )message.getID() );
            BufferedReader    bufferedreader = new BufferedReader( new InputStreamReader( fileinputstream ) );

            message.setReader( bufferedreader );   // let user read message

            /*
             * StringBuffer      newMsg = new StringBuffer();
             * String line = null;
             * while (( line = bufferedreader.readLine() ) != null)
             * {
             * newMsg.append( line ).append( "\n" );
             * }
             * fileinputstream.close();
             * bufferedreader.close();
             * message.setMessage( newMsg.toString() );
             */

            msgOK = true;
         }
      }
      catch ( Exception x )
      {

         // file related problem - ignore it here !

         handleException( x );
      }

      return msgOK;
   }


   /**
    * returns the number of Messages in persistant storage for a spezified user
    * @param username
    * @long number of messages
    */
   public long getNrOfMessages( String user )
   {
      File  file = new File( getConfiguration().getHome(), user );

      if ( !file.exists() )
      {
         file.mkdirs();

         if ( Debug.enabled == true )
         {
            writeToLog( "MessageHandlerFile Created directories = " + file.toString() );
         }
      }

      return file.list().length;
   }


   /**
    * save MailMessage to file in directory spezified by path
    * @param full path containig username
    * @param message
    * @throws Exception
    */
   final private static void writeMessage( String path, String msg ) throws Exception
   {
      FileOutputStream  fileoutputstream = null;
      PrintWriter       printwriter = null;

      try
      {
         fileoutputstream = new FileOutputStream( path );
         printwriter = new PrintWriter( fileoutputstream );

         printwriter.println( msg );
         printwriter.flush();
      }
      catch ( Exception x )
      {
         throw x;
      }
      finally
      {
         if ( printwriter != null )
         {
            printwriter.close();
         }

         if ( fileoutputstream != null )
         {
            fileoutputstream.close();
         }
      }
   }


   /**
    * delete file that represents a spezified MailMessage
    * @param MailMessage
    */
   final public void deleteMessage( String messageId )
   {
      try
      {
         File     file = new File( messageId );
         boolean  rc = file.delete();

         if ( Debug.enabled )
         {
            if ( rc == false )
            {
               writeToLog( "MessageHandler failed to delete file : " + file.getAbsolutePath() );
            }
         }
      }
      catch ( Exception x )
      {
         handleException( x );
      }
   }


   /**
    * Method declaration
    *
    *
    * @param Message
    *
    *
    */
   final public void deleteMessage( MailMessage Message )
   {
      deleteMessage( ( String )Message.getID() );
   }


   /**
    * Method declaration
    *
    *
    * @param messageId
    *
    *
    * @return
    *
    */
   public boolean existsMessage( String messageId )
   {
      boolean  valid = false;

      try
      {
         String   filename = getConfiguration().getHome() + File.separator + Sender.getOutgoingMailQueueName() + File.separator + messageId + ".msg";
         File     file = new File( filename );

         valid = file.isFile();
      }
      catch ( Exception x )
      {

         // intentionally ignored

      }

      return valid;
   }


}



/*--- formatting done in "JPS Convention" style on 08-20-2001 ---*/

