/*--- formatted by Jindent 2.1, (www.c-lab.de/~jindent) ---*/

/*
 * Copyright (c) 1999 / 2000  Johannes Plachy & Computer Mutter
 *
 * JMailSrv@jps.at
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * If you modify this file, please send us a copy.
 */

/********************************************************************************************
* CVS Info
*
* $Source: /cvsroot/jmailsrv/jmailserver/at/jps/mailserver/MessageHandlerJDBC.java,v $
*
* $Author: furykid $
* $Date: 2002/06/21 20:46:44 $
*
* modification history:
*
* $Log: MessageHandlerJDBC.java,v $
* Revision 1.5  2002/06/21 20:46:44  furykid
* bug fix
*
* Revision 1.4  2002/01/20 16:07:13  furykid
* added new header for cvs information
*

*********************************************************************************************/


package at.jps.mailserver;

import java.io.*;
import java.sql.*;
import java.util.*;
import at.jps.mailserver.util.*;

// needed Tables : Messages : ID MSG SIZE DATE USER
// examples:
// JDBCDriver="org.gjt.mm.mysql.Driver"
// JDBCServerURL="jdbc:mysql://www.jps.at/"
// JDBCPort="3306"
// JDBCDatabase="Databasename"
// JDBCUser="Username"
// JDBCPassword="Password"

/**
 * This class handles the mailmessages stored in a JDBC accessable database
 * @author  Johannes Plachy
 * @version 1.100, 11/22/99
 */
public class MessageHandlerJDBC extends MessageHandlerImpl implements MessageHandler, CFGUpdateable
{
   private ConfigurationManager  ivConfigurationManager;

   private PreparedStatement     ivPstmtCount = null;
   private PreparedStatement     ivPstmtAllEmpty = null;
   private PreparedStatement     ivPstmtDelete = null;
   private PreparedStatement     ivPstmtInsert = null;
   private PreparedStatement     ivPstmtOneMsg = null;

   /**
    * returns configuration file reader
    * @return configuration file reader
    */
   final private ConfigurationManager getConfiguration()
   {
      return ivConfigurationManager;
   }


   /**
    * update configuration file reader
    */
   final public void updateCFG( ConfigurationManager ConfigurationManager )
   {
      ivConfigurationManager = ConfigurationManager;
   }



   /**
    * Constructor gets JDBC driver class and the URL to access the server from the configuration file
    * and tries to instantiate the driver,
    * @param ConfigurationManager to access the configuration information
    */
   MessageHandlerJDBC( ConfigurationManager ConfigurationManager )
   {
      updateCFG( ConfigurationManager );

      try
      {
         /*
          * ivJDBCURLStr = getConfiguration().getJDBCServerURL()+":"+getConfiguration().getJDBCPort()+"/"
          * +getConfiguration().getJDBCDatabase()
          * +"?user="+getConfiguration().getJDBCUser()
          * +"&password="+getConfiguration().getJDBCPassword()
          * +"&autoReconnect=true";
          */

      }
      catch ( Throwable throwable )
      {
         handleException( throwable );
      }
   }


   /**
    * establish a connection to the databaseserver and prepares the most often used SQL commands.
    * @return true if a connection could be established
    */
   public void initialize()
   {
      try
      {
         Connection ivSQLconnection = JDBCConfig.getConnection( getConfiguration().getJDBCDriver(), getConfiguration().getJDBCURL() );

         // count commando

         ivPstmtCount = ivSQLconnection.prepareStatement( "SELECT COUNT(*) FROM MESSAGES WHERE USER=?" );

         // get all messages from user commando

         ivPstmtAllEmpty = ivSQLconnection.prepareStatement( "SELECT ID,SIZE,USER FROM MESSAGES WHERE USER=?" );

         // get only one messagebody

         ivPstmtOneMsg = ivSQLconnection.prepareStatement( "SELECT MSG FROM MESSAGES WHERE USER=? AND ID=?" );

         // insert commando

         ivPstmtInsert = ivSQLconnection.prepareStatement( "INSERT INTO MESSAGES (MSG,SIZE,USER,DATE) VALUES (?,?,?,?)" );

         // delete commando

         ivPstmtDelete = ivSQLconnection.prepareStatement( "DELETE FROM MESSAGES WHERE ID=? AND USER=?" );

      }
      catch ( Exception x )
      {
         handleException( x );
      }

   }

   /**
   * initialize Handler - in this case open connection to database server
   */
   private void reinitialize()
   {
       if (isConnected() == false)
       {
           initialize();
       }
   }


   /**
    * is alway connected
    * @return always true
    */
   public boolean isConnected()
   {
      return JDBCConfig.isConnected();
   }



   /**
    * stores a given mailmessage to the database
    * @param tempFilename
    * @param user   for which to store the message
    * @param ExternalUsers comma seperated list of external users - might be null
    */
   public void saveMessage( String user, String message )
   {
      super.saveMessage( user, message );

      try
      {
         reinitialize();

         ivPstmtInsert.clearParameters();
         ivPstmtInsert.setString( 1, message );
         ivPstmtInsert.setLong( 2, message.length() );
         ivPstmtInsert.setString( 3, user );
         ivPstmtInsert.setDate( 4, new java.sql.Date( System.currentTimeMillis() ) );
         ivPstmtInsert.executeUpdate();
      }
      catch ( SQLException sqlx )
      {
         handleException( sqlx );
      }
   }


   /**
    * Method declaration
    * @param tempFilename
    * @param user
    * @param ExternalUsers
    */
   public void saveMessage( String tempFilename, String user, String sender, Vector ExternalUsers )
   {
      super.saveMessage( tempFilename, user, ExternalUsers );

      try
      {

         reinitialize();

         BufferedReader reader = new BufferedReader( new FileReader( tempFilename ) );
         StringBuffer   msg = new StringBuffer();

         if ( ( ExternalUsers != null ) && ( ExternalUsers.size() > 0 ) )
         {
            Iterator iter = ExternalUsers.iterator();

            if ( sender != null )
            {
               msg.append( sender ).append( "\n" );
            }

            while ( iter.hasNext() )
            {
               msg.append( ( String )iter.next() );
            }

            msg.append( "\n" );
         }

         try
         {
            String   line = null;

            do
            {
               line = reader.readLine();

               if ( line != null )
               {
                  msg.append( line).append( "\n" );
               }
            }
            while ( line != null );
         }
         catch ( Throwable t ) {}
         finally
         {
            if ( reader != null )
            {
               try
               {
                  reader.close();
               }
               catch ( Exception x )
               {

                  // x

               }
            }
         }

         try
         {
            ivPstmtInsert.clearParameters();
            ivPstmtInsert.setString( 1, msg.toString() );
            ivPstmtInsert.setLong( 2, msg.length() );
            ivPstmtInsert.setString( 3, user );
            ivPstmtInsert.setDate( 4, new java.sql.Date( System.currentTimeMillis() ) );
            ivPstmtInsert.executeUpdate();
         }
         catch ( SQLException sqlx )
         {
            handleException( sqlx );
         }
      }
      catch ( Exception x )
      {
         handleException( x );
      }
   }


   /**
    * retrieve a MailMessage based on a passed Id
    * @param id that identifies a message uniquely
    * @return MailMessage that has been retrieved
    * @see MailMessage
    */
   public MailMessage getOutgoingMessage( String id )
   {

      // NYI

      return null;
   }


   /**
    * returns an array of MailMessages with information that is used only to satisfy the Mailclient requests
    * information about size and amount
    * @param user for which messages are to be retrieved
    * @return MailMessage[] Array of Messages available for retrival ( shallow)
    */
   final public MailMessage[] getMessages( String user )
   {
      int nrOfRecords = ( int )getNrOfMessages( user );

      if ( nrOfRecords < 1 )
      {
         return null;
      }

      try
      {
         reinitialize();


         ivPstmtAllEmpty.clearParameters();
         ivPstmtAllEmpty.setString( 1, user );
         ivPstmtAllEmpty.execute();

         ResultSet      rs = ivPstmtAllEmpty.getResultSet();
         MailMessage[]  messages = new MailMessage[ nrOfRecords ];
         int            i = 0;

         while ( rs.next() )
         {
            messages[ i ] = new MailMessage();

            messages[ i ].setID( rs.getObject( 1 ) );
            messages[ i ].setSize( rs.getLong( 2 ) );
            messages[ i ].setUser( rs.getString( 3 ) );

            i++;
         }

         /*
          * for (int i = 0; i < nrOfRecords; i++)
          * {
          * rs.next();
          * }
          */

         return messages;
      }
      catch ( Throwable throwable )
      {
         handleException( throwable );

         return null;
      }
   }


   /**
    * retrieve next (if any) available MailMessage
    * @param initialized MailMessage needed to retrieve the Mailcontents itself
    * @return true if Message is available and could be retrieved
    */
   public boolean getNextMessage( MailMessage message )
   {
      if ( ( message != null ) && ( message.isFetched() ) )
      {
         return true;
      }

      boolean  msgOK = false;

      try
      {

         reinitialize();

         if ( ( message != null ) && ( message.isFetched() != true ) )
         {

            // retrieve Msg from database

            ivPstmtOneMsg.clearParameters();
            ivPstmtOneMsg.setString( 1, message.getUser() );

            Integer  i = ( Integer )message.getID();

            ivPstmtOneMsg.setInt( 2, i.intValue() );
            ivPstmtOneMsg.execute();

            ResultSet   rs = ivPstmtOneMsg.getResultSet();

            // use rs.getBinaryStream !!

            message.setMessage( rs.getString( 1 ) );     // (String)rs.getObject(1));

            ByteArrayInputStream inputstream = new ByteArrayInputStream( message.getMessage().getBytes() );
            BufferedReader       bufferedreader = new BufferedReader( new InputStreamReader( inputstream ) );

            message.setReader( bufferedreader );

            msgOK = true;
         }
      }
      catch ( Throwable throwable )
      {
         handleException( throwable );
      }

      return msgOK;
   }


   /**
    * returns the number of existing messages for a given user
    * @param user for which messages are to be retrieved
    * @return number of pending messages
    */
   public long getNrOfMessages( String user )
   {

      reinitialize();

      try
      {
         ivPstmtCount.clearParameters();
         ivPstmtCount.setString( 1, user );
         ivPstmtCount.execute();

         ResultSet   rs = ivPstmtCount.getResultSet();

         if ( rs.next() )
         {
            return rs.getInt( 1 );
         }
         else
         {
            return 0;
         }
      }
      catch ( SQLException sqlx )
      {
         handleException( sqlx );
      }

      return -1;
   }


   /**
    * delete given Message from database
    * @param MailMessage to be removed from database
    */

   // user+Id

   final public void deleteMessage( String usernamePlusmessageId )
   {
      try
      {
         reinitialize();

         StringTokenizer   tokenizer = new StringTokenizer( usernamePlusmessageId, "+" );
         String            userName = tokenizer.nextToken();
         String            messageId = tokenizer.nextToken();

         ivPstmtDelete.clearParameters();

         int   i = Integer.parseInt( messageId );

         ivPstmtDelete.setInt( 1, i );
         ivPstmtDelete.setString( 2, userName );
         ivPstmtDelete.executeUpdate();
      }
      catch ( Throwable throwable )
      {
         handleException( throwable );
      }
   }


   /**
    * Method declaration
    *
    *
    * @param Message
    *
    *
    */
   final public void deleteMessage( MailMessage Message )
   {
      deleteMessage( Message.getUser() + "+" + Integer.toString( ( ( Integer )Message.getID() ).intValue() ) );
   }


   /**
    * Method declaration
    *
    *
    * @param messageId
    *
    *
    * @return
    *
    */
   public boolean existsMessage( String messageId )
   {

      // NYI !!!!

      return true;
   }


}



/*--- formatting done in "JPS Convention" style on 08-20-2001 ---*/

