/*--- formatted by Jindent 2.1, (www.c-lab.de/~jindent) ---*/

/*
 * Copyright (c) 1999 / 2000  Johannes Plachy & Computer Mutter
 *
 * JMailSrv@jps.at
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * If you modify this file, please send us a copy.
 */

/********************************************************************************************
* CVS Info
*
* $Source: /cvsroot/jmailsrv/jmailserver/at/jps/mailserver/OutgoingMessagesManager.java,v $
*
* $Author: furykid $
* $Date: 2003/04/01 20:35:57 $
*
* modification history:
*
* $Log: OutgoingMessagesManager.java,v $
* Revision 1.3  2003/04/01 20:35:57  furykid
* cleanup
*
* Revision 1.2  2002/01/20 16:07:13  furykid
* added new header for cvs information
*

*********************************************************************************************/


package at.jps.mailserver;

import java.io.*;
import java.util.Vector;
import java.util.zip.GZIPInputStream;
import java.util.zip.GZIPOutputStream;

/**
 * Class declaration
 * @author Johannes Plachy
 * @version %I%, %G%
 */
class OutgoingMessagesManager
{
   final private static String            FILENAME = "jms.omi";
   private static OutgoingMessagesManager cvInstance;
   private Vector                         ivMessagesToSend;
   private ConfigurationManager           ivConfigurationManager;

   /**
    * Constructor declaration
    * @param ConfigurationManager
    */
   private OutgoingMessagesManager( ConfigurationManager configurationManager )
   {
      updateCFG( configurationManager );
      initialize();

      // now if there are pending messages
      // send signal

      if ( ivConfigurationManager.isSenderRunning() )
      {
         trigggerProcessing();
      }
      else
      {
         if ( Debug.enabled == true )
         {
            ErrorReporter.getInstance().writeSenderMsg( "Outgoing Mail ! but no sender : " + ivMessagesToSend.size() );
         }
      }
   }


   /**
    * this method will trigger an event to allow the sender thread start deliver pending outgoing mails,
    * will be called during initialization phase but also from sender if manually started
    */
   public void trigggerProcessing()
   {
      if ( ivMessagesToSend.size() > 0 )
      {
         ivConfigurationManager.getSender().processMail();

         if ( Debug.enabled == true )
         {
            ErrorReporter.getInstance().writeSenderMsg( "Outgoing Mail ! sender triggered : " + ivMessagesToSend.size() );
         }
      }
   }


   /**
    * Method declaration
    * @param ConfigurationManager
    * @return
    */
   public static OutgoingMessagesManager getInstance( ConfigurationManager configurationManager )
   {
      if ( cvInstance == null )
      {
         cvInstance = new OutgoingMessagesManager( configurationManager );
      }

      return cvInstance;
   }


   /**
    * Method declaration
    * @return
    */
   final private ConfigurationManager getConfiguration()
   {
      return ivConfigurationManager;
   }


   /**
    * update configuration information
    * @return new configuration information
    */
   public void updateCFG( ConfigurationManager ConfigurationManager )
   {
      ivConfigurationManager = ConfigurationManager;
   }


   /**
    * Method declaration
    */
   private void initialize()
   {

      // try to load settings from file

      String   filename = getConfiguration().getHome() + File.separator + FILENAME;
      File     fileTest = new File( filename );

      if ( fileTest.isFile() == true || fileTest.exists() == true )
      {

         // ok ... there is a save file - try to do the deserialization

         ObjectInputStream in = null;

         try
         {
            in = new ObjectInputStream( new GZIPInputStream( new FileInputStream( filename ) ) );
            ivMessagesToSend = ( Vector )in.readObject();
         }
         catch ( Exception x )
         {
            ErrorReporter.getInstance().writeSenderMsg( "MessageInformationfile <" + filename + "> could not be read: " + x.toString() );
         }
         finally
         {
            if ( in != null )
            {
               try
               {
                  in.close();
               }
               catch ( Exception x ) {}
            }
         }

         // now check for valid entries

         Vector         validatedEntries = new Vector( 100 );
         MessageHandler messagehandler = ivConfigurationManager.getMessageHandler();

         for ( int i = 0; i < ivMessagesToSend.size(); i++ )
         {
            MessageInfo messageInfo = ( MessageInfo )ivMessagesToSend.get( i );

            // check if corresponding message is still available
            // might be physically deleted by external administrativ
            // activities !?!?

            if ( messagehandler.existsMessage( messageInfo.getMessageId() ) )
            {
               validatedEntries.add( messageInfo );
            }
            else
            {
               if ( Debug.enabled == true )
               {
                  ErrorReporter.getInstance().writeSenderMsg( "Outgoing Mail Information discarted : " + messageInfo.getMessageId() );
               }
            }
         }

         // now reset messagevector !

         ivMessagesToSend = null;
         ivMessagesToSend = validatedEntries;
      }
      else
      {
         ivMessagesToSend = new Vector();
      }
   }


   /**
    * Method declaration
    * @param messageId
    * @param receivers
    */
   void addMessage( String messageId, Vector receivers )
   {
      ivMessagesToSend.add( new MessageInfo( messageId, receivers ) );
      saveInformation();
   }


   /**
    * Method declaration
    * @param messageInfo
    */
   void removeMessage( MessageInfo messageInfo )
   {
      ivMessagesToSend.removeElement( messageInfo );
   }


   /**
    * Method declaration
    * @return
    */
   Vector getMessages()
   {
      saveInformation();

      return ivMessagesToSend;
   }


   /**
    * Method declaration
    */
   void saveInformation()
   {
      String               filename = getConfiguration().getHome() + File.separator + FILENAME;
      ObjectOutputStream   out = null;

      try
      {
         out = new ObjectOutputStream( new GZIPOutputStream( new FileOutputStream( filename ) ) );

         out.writeObject( ivMessagesToSend );
      }
      catch ( Exception x )
      {
         ErrorReporter.getInstance().writeSenderMsg( "MessagaeInformationfile <" + filename + "> could not be written: " + x.toString() );
      }
      finally
      {
         try
         {
            if ( out != null )
            {
               out.close();
            }
         }
         catch ( Exception x ) {}
      }
   }


}



/*--- formatting done in "JPS Convention" style on 08-20-2001 ---*/

