/*--- formatted by Jindent 2.1, (www.c-lab.de/~jindent) ---*/

/*
 * Copyright (c) 1999 / 2000  Johannes Plachy & Computer Mutter
 *
 * JMailSrv@jps.at
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * If you modify this file, please send us a copy.
 */

/********************************************************************************************
* CVS Info
*
* $Source: /cvsroot/jmailsrv/jmailserver/at/jps/mailserver/Pop3Connection.java,v $
*
* $Author: furykid $
* $Date: 2003/05/29 18:04:47 $
*
* modification history:
*
* $Log: Pop3Connection.java,v $
* Revision 1.3  2003/05/29 18:04:47  furykid
* rebuild 290503
*
* Revision 1.2  2002/01/20 16:07:13  furykid
* added new header for cvs information
*

*********************************************************************************************/


package at.jps.mailserver;

import java.io.*;
import java.net.Socket;
import java.util.Date;
import java.util.Locale;
import java.util.StringTokenizer;
import java.text.DateFormat;

/**
 * Pop3Connection Every connection to the pop3 server is hidden by an pop3connection.
 * The server create for every request an Thread and starts an Pop3Connection class.
 * @author (c) 1998 Computer Mutter computer@mutter.com ; 1999 by Johannes Plachy
 * @version 0.99 11/1999
 */
final public class Pop3Connection implements Runnable, CFGUpdateable
{
   final private static int      _SEND_TIMEOUT = 300000;

   /**
    * socket for communication
    */
   private Socket                ivClientSocket;

   /**
    * reader for inputstream
    */
   private BufferedReader        ivInputStream;

   /**
    * writer for outputstream
    */
   private PrintWriter           ivOutputStream;

   /**
    * flag that indicates if the platform needs an additional CR after each line
    */
   private boolean               ivbAddCR;

   /**
    * Array of Mails that are currently processed
    */
   private MailMessage           ivMessages[] = null;

   /**
    * configuration data
    */
   private ConfigurationManager  ivConfigurationManager;

   /**
    * creates a new connectionhandler thread that handles a single connection
    * @param configuration data
    * @param communication socket
    */
   public Pop3Connection( ConfigurationManager ConfigurationManager, Socket socket )
   {
      updateCFG( ConfigurationManager );

      ivbAddCR = ( System.getProperty( "line.separator" ).indexOf( "\r" ) == -1 );

      setSocket( socket );

      try
      {
         setInputStream( new BufferedReader( new InputStreamReader( getSocket().getInputStream() ) ) );
         setOutputStream( new PrintWriter( new BufferedWriter( new OutputStreamWriter( getSocket().getOutputStream() ) ) ) );
      }
      catch ( Exception exception )
      {
         handleException( exception );
      }
   }


   /**
    * Method declaration
    *
    *
    * @return
    *
    */
   static int getDefaultTimeout()
   {
      return _SEND_TIMEOUT;
   }


   /**
    * retrieve configuration information
    * @return configuration data
    */
   final private ConfigurationManager getConfiguration()
   {
      return ivConfigurationManager;
   }


   /**
    * new configuration info has to be used
    * @param new configuration data
    */
   public void updateCFG( ConfigurationManager ConfigurationManager )
   {
      ivConfigurationManager = ConfigurationManager;
   }


   /**
    * test if the wanted mail is in the range of existing ones
    * @param index to e checked
    * @return true if mail is available
    */
   final private boolean checkindex( int i )    // index starts with 1 !!
   {
      if ( ( i <= 0 ) || i > ( getMsgCount() + 1 ) )
      {
         println( "-ERR no such message, only " + getMsgCount() + " messages in maildrop", true );

         return false;
      }

      return true;
   }


   /**
    * POP3 protocoll specific handler delete request
    * @param index of mails
    */
   final private void cmdDELE( int i )    // index starts at 1 !!
   {
      if ( checkindex( i ) == false )
      {
         return;
      }

      getConfiguration().getMessageHandler().deleteMessage( ivMessages[ i - 1 ] );
      println( "+OK message " + i + " deleted", true );
   }


   /**
    * POP3 protocoll specific handler list request
    */
   final private void cmdLIST()
   {
      println( "+OK " + getMsgCount() + " messages", true );

      for ( int i = 0; i < getMsgCount(); i++ )
      {
         println( ( i + 1 ) + " " + ivMessages[ i ].getSize(), true );
      }

      println( ".", true );
   }


   /**
    * POP3 protocoll specific handler list request
    * @param index of mails
    */
   final private void cmdLIST( int i )    // index starts with 1 !!!!
   {
      if ( checkindex( i ) == false )
      {
         return;
      }

      println( "+OK " + i + " " + ivMessages[ i - 1 ].getSize(), true );
   }


   /**
    * POP3 protocoll specific handler retrieve request
    * @param index of mails
    * @param number of lines
    */
   final private void cmdRETR( int idx ) throws Exception
   {
      if ( checkindex( idx ) == false )
      {
         return;
      }

      println( "+OK " + ivMessages[ idx - 1 ].getSize() + " octets", true );

      if ( getConfiguration().getMessageHandler().getNextMessage( ivMessages[ idx - 1 ] ) )
      {

         // ByteArrayInputStream inputstream = new ByteArrayInputStream( ivMessages[ idx - 1 ].getMessage().getBytes() );
         // BufferedReader       bufferedreader = new BufferedReader( new InputStreamReader( inputstream ) );

         BufferedReader bufferedreader = ivMessages[ idx - 1 ].getReader();
         String         line = null;

         while ( ( line = bufferedreader.readLine() ) != null )
         {
            if ( line != null )
            {
               if ( Debug.enabled )
               {
                  System.out.println( line );
               }

               println( line, false );
            }
         }

         bufferedreader.close();
         println( ".", true );
      }
      else
      {
         throw new Exception( " Message (" + idx + ") not found" );
      }
   }


   /**
    * returns the total size of all messages that are ready for retrival
    * @return size of all messages
    */
   final private long getMaildropSize()
   {
      return getMsgTotalSize();
   }


   /**
    * POP3 protocoll specific handler statistic request
    * @param username
    */
   final private void cmdSTAT( ) //String user )
   {
      println( "+OK " + getMsgCount() + " " + Long.toString( getMaildropSize() ), true );
   }


   /**
    * Reader for inputstream
    * @return reader for socket communication
    */
   final private BufferedReader getInputStream()
   {
      return ivInputStream;
   }


   /**
    * Writer for outputstream
    * @return writer for socket communication
    */
   final private PrintWriter getOutputStream()
   {
      return ivOutputStream;
   }


   /**
    * get communication socket
    * @return socket for communication
    */
   final private Socket getSocket()
   {
      return ivClientSocket;
   }


   /**
    * get the list of all mailusers
    * @return list of users
    */
   final private UserList getUserList()
   {
      return getConfiguration().getUserList();
   }


   /**
    * writes exception information to the logfile
    * @param exception to be logged
    */
   final private void handleException( Throwable throwable )
   {
      writeErrorMsg( throwable.toString() );
   }


   /**
    * writes line to socket outputstream
    * @param line of a Email
    */
   final private void println( String s, boolean flushit )
   {
      PrintWriter printwriter = getOutputStream();

      // damned lineseperator causes problems to heterogen environments

      if ( ivbAddCR )
      {
         printwriter.println( s + "\r" );    // linux version
      }
      else
      {
         printwriter.println( s );           // windows version
      }

      if ( flushit )
      {
         printwriter.flush();
      }

      if ( Debug.enabled )
      {
         ErrorReporter.getInstance().writePop3Msg( s );
      }
   }


   /**
    * get number of messages that can be retrieved
    * @return number of messages
    */
   final private long getMsgCount()
   {
      return ( ivMessages != null ) ? ( ivMessages.length ) : 0;
   }


   /**
    * get total size of messages that can be retrieved
    * @return size of messages
    */
   final private long getMsgTotalSize()
   {
      long  sizes = 0;

      if ( ivMessages != null )
      {
         for ( int i = 0; i < ivMessages.length; i++ )
         {
            sizes += ivMessages[ i ].getSize();
         }
      }

      return sizes;
   }


   /**
    * get messages for a specific user this messages are only fetched shallow !
    * @param username
    * @see MailMessage
    */
   final private void retrieveMsgInfo( String user )
   {
      ivMessages = getConfiguration().getMessageHandler().getMessages( user );
   }


   /**
    * loop that handles local requests for mailtransfer
    */
   final public void run()
   {
      try
      {
         DateFormat  dateformat = DateFormat.getDateTimeInstance( DateFormat.LONG, DateFormat.LONG, Locale.getDefault() );

         println( "+OK Java POP3 Server ready. (" + dateformat.format( new Date() ) + ")", true );

         boolean  flag = true;
         String   user = "";

         do
         {
            String   input;

            try
            {
               input = getInputStream().readLine();

               if ( input == null )
               {
                  break;
               }
            }
            catch ( Exception exception1 )
            {
               handleException( exception1 );

               break;
            }

            StringTokenizer   stringtokenizer = new StringTokenizer( input );
            String            command;

            try
            {
               command = stringtokenizer.nextToken();
               command = command.toUpperCase();
            }
            catch ( Exception _ex )
            {
               continue;
            }

            if ( !flag )
            {
               if ( command.compareTo( "DELE" ) == 0 )
               {
                  try
                  {
                     cmdDELE( Integer.parseInt( stringtokenizer.nextToken() ) );
                  }
                  catch ( Exception _ex )
                  {
                     println( "-ERR no such message", true );
                  }

                  continue;
               }

               if ( command.compareTo( "TOP" ) == 0 )
               {
                  cmdRETR( Integer.parseInt( stringtokenizer.nextToken() )); //, 0 );

                  continue;
               }

               if ( command.compareTo( "NOOP" ) == 0 )
               {
                  println( "+OK Ok.", true );

                  continue;
               }

               if ( command.compareTo( "UIDL" ) == 0 )
               {
                  println( "-Err unknown command <" + command + ">", true );

                  continue;
               }

               if ( command.compareTo( "STAT" ) == 0 )
               {
                  cmdSTAT(); // user );

                  continue;
               }

               if ( command.compareTo( "LIST" ) == 0 )
               {
                  try
                  {
                     cmdLIST( Integer.parseInt( stringtokenizer.nextToken() ) );
                  }
                  catch ( Exception _ex )
                  {
                     cmdLIST();
                  }

                  continue;
               }

               if ( command.compareTo( "RETR" ) == 0 )
               {
                  try
                  {
                     cmdRETR( Integer.parseInt( stringtokenizer.nextToken() )); //, 0 );
                  }
                  catch ( Exception _ex )
                  {
                     println( "-ERR no such message", true );
                  }

                  continue;
               }
            }
            else
            {
               if ( command.compareTo( "PASS" ) == 0 )
               {
                  try
                  {
                     String   pwd = stringtokenizer.nextToken();

                     if ( getUserList().getPassword( user.toLowerCase() ).equalsIgnoreCase( pwd ) )
                     {
                        flag = false;

                        retrieveMsgInfo( user );

                        // println("+OK User successfully logged on",true);

                        println( "+OK Maildrop has " + getMsgCount() + " messages (" + Long.toString( getMaildropSize() ) + " octets)", true );

                        if ( Debug.enabled )
                        {
                           writeStatusMsg( user + " connected" );
                        }
                     }
                     else
                     {
                        println( "-ERR Wrong password.", true );
                        writeErrorMsg( "Wrong password for user " + user );
                     }
                  }
                  catch ( Exception _ex )
                  {
                     println( "-ERR Unkown User.", true );
                  }

                  continue;
               }

               if ( command.compareTo( "AUTH" ) == 0 )
               {
                  println( "-Err unknown command <" + command + ">", true );

                  continue;
               }

               if ( command.compareTo( "USER" ) == 0 )
               {
                  try
                  {
                     user = stringtokenizer.nextToken().toLowerCase();

                     if ( getUserList().contains( user ) )
                     {
                        println( "+OK Password required for " + user, true );
                     }
                     else
                     {
                        println( "-ERR Unkown User.", true );
                        writeErrorMsg( "Unkown User " + user );
                     }
                  }
                  catch ( Exception _ex )
                  {
                     println( "-ERR Unkown User.", true );
                  }

                  continue;
               }
            }

            if ( command.compareTo( "QUIT" ) == 0 )
            {
               println( "+OK Java POP3 Server closing connection", true );

               if ( Debug.enabled )
               {
                  writeStatusMsg( user + " disconnected" );
               }

               break;
            }

            println( "-Err unknown command <" + command + ">", true );

            if ( Debug.enabled )
            {
               writeStatusMsg( "Unknown command: " + command );
            }
         }
         while ( true );
      }
      catch ( Throwable exception )
      {
         handleException( exception );
      }
      finally
      {
         try
         {
            getSocket().close();
         }
         catch ( Exception x ) {}
      }
   }


   /**
    * set inputstream for socket communication
    * @param Reader for inputstream
    */
   final private void setInputStream( BufferedReader bufferedreader )
   {
      ivInputStream = bufferedreader;
   }


   /**
    * set outputstream for socket communication
    * @param Writer for outputstream
    */
   final private void setOutputStream( PrintWriter printwriter )
   {
      ivOutputStream = printwriter;
   }


   /**
    * set socket for communication
    * @param new open socket
    */
   final private void setSocket( Socket socket )
   {
      ivClientSocket = socket;

      /*
       * try
       * {
       * ivClientSocket.setSoTimeout( getConfiguration().getPOP3Timeout() );
       * }
       * catch ( java.net.SocketException exception )
       * {
       * handleException( exception );
       * }
       */

   }


   /**
    * write (severe) message to logfile
    * @param line to be written
    */
   final private void writeErrorMsg( String s )
   {
      ErrorReporter.getInstance().writePop3Msg( s, "Error" );
   }


   /**
    * write message to logfile
    * @param line to be written
    */
   final private void writeStatusMsg( String s )
   {
      ErrorReporter.getInstance().writePop3Msg( s );
   }


}



/*--- formatting done in "JPS Convention" style on 08-20-2001 ---*/

