/*--- formatted by Jindent 2.1, (www.c-lab.de/~jindent) ---*/

/*
 * Copyright (c) 1999 / 2000  Johannes Plachy & Computer Mutter
 *
 * JMailSrv@jps.at
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * If you modify this file, please send us a copy.
 */

/********************************************************************************************
* CVS Info
*
* $Source: /cvsroot/jmailsrv/jmailserver/at/jps/mailserver/Pop3Server.java,v $
*
* $Author: furykid $
* $Date: 2003/04/01 20:35:55 $
*
* modification history:
*
* $Log: Pop3Server.java,v $
* Revision 1.3  2003/04/01 20:35:55  furykid
* cleanup
*
* Revision 1.2  2002/01/20 16:07:13  furykid
* added new header for cvs information
*

*********************************************************************************************/


package at.jps.mailserver;

import java.io.IOException;
import java.net.ServerSocket;

/**
 * Pop3Server Creates an ServerSocket on port 110 and waits for clients to be connected.
 * Then starts for every connection an Thread with an Pop3Connection class.
 * The Server looks in the local directory for an user and sends the contents to it.
 * The User must be exist to have access to the certain data. This can be executed as an application !
 * @author (c) 1998 Computer Mutter computer@mutter.com ; adapted 1999 by Johannes Plachy
 * @version 0.99 11/99
 */
final public class Pop3Server extends Thread implements CFGUpdateable
{

   /**
    * Serversocket
    */
   private ServerSocket          ivPop3Socket;

   /**
    * Connectionhandler for a POP3 connection
    */
   private Pop3Connection        ivPop3Connection;

   /**
    * Configuration data
    */
   private ConfigurationManager  ivConfigurationManager;

   /**
    * default TCPIP port = 110
    */
   final private static int      DEFAULT_ServerPort = 110;

   /**
    * create new POP server with configuration info
    * @param configuration data
    */
   public Pop3Server( ConfigurationManager configurationManager )
   {
      updateCFG( configurationManager );

      try
      {
         setPop3Socket( new ServerSocket( getConfiguration().getLocalPop3ServerPort() ) );
      }
      catch ( Exception exception )
      {
         handleException( exception, "error cannot start Pop3 Server" );

         return;

         // System.exit(1); // dont exit other threads might be still alive !

      }

      start();
   }


   /**
    * get configuration info
    * @return configuration data
    */
   final private ConfigurationManager getConfiguration()
   {
      return ivConfigurationManager;
   }


   /**
    * get default port
    * @return TCP/IP port
    */
   final public static int getDefaultPop3Port()
   {
      return DEFAULT_ServerPort;
   }


   /**
    * get open socket
    * @return TCP/IP socket
    */
   final private ServerSocket getPop3Socket()
   {
      return ivPop3Socket;
   }


   /**
    * write Exception information to logfile
    * @param exception
    * @param more information about exception
    */
   final private void handleException( Throwable throwable, String Comment )
   {
      writeErrorMsg( throwable.toString() + " " + Comment );

      if ( Debug.enabled )
      {
         throwable.printStackTrace();
      }
   }


   /**
    * wait for connections and fork of connection handlers
    */
   final public void run()
   {
      try
      {
         do
         {
            java.net.Socket   socket;

            try
            {
               if ( Debug.enabled )
               {
                  writeStatusMsg( "POP3 Server waiting for connection..." );
               }

               socket = getPop3Socket().accept();
            }
            catch ( IOException ioexception )
            {
               handleException( ioexception, "error checking port !" );

               break;
            }

            if ( Debug.enabled )
            {
               writeStatusMsg( "POP3 Server starting new POP3 Connectionhandler" );
            }

            ivPop3Connection = new Pop3Connection( getConfiguration(), socket );

            if ( ivPop3Connection != null )
            {
               getConfiguration().getThreadPool().addTask( ivPop3Connection );
            }
            else
            {
               writeErrorMsg( "ivPop3Connection == null <- error" );
            }
         }
         while ( true );
      }
      catch ( Throwable exception )
      {
         handleException( exception, "Exception happened in POP3 run Loop" );
      }

   }


   /**
    * save serverside socket for internal use
    * @param serversocket
    */
   private void setPop3Socket( ServerSocket serversocket )
   {
      ivPop3Socket = serversocket;
   }


   /**
    * configuration information has changed
    * @param new configuration data
    */
   public void updateCFG( ConfigurationManager ConfigurationManager )
   {
      ivConfigurationManager = ConfigurationManager;

      if ( ivPop3Connection != null )
      {
         ivPop3Connection.updateCFG( getConfiguration() );
      }
   }


   /**
    * write (severe) message to logfile
    * @param line to be written
    */
   final private void writeErrorMsg( String s )
   {
      ErrorReporter.getInstance().writePop3Msg( s, "Error" );
   }


   /**
    * write message to logfile
    * @param line to be written
    */
   final private void writeStatusMsg( String s )
   {
      ErrorReporter.getInstance().writePop3Msg( s );
   }


   /**
    * helper function to instatiate a POP server
    * @param configuration
    */
   public static Pop3Server startServer( ConfigurationManager ConfigurationManager )
   {
      return new Pop3Server( ConfigurationManager );
   }


   /**
    * Method declaration
    * @param args
    */
   public static void main( String args[] )
   {
      System.out.println( "Java Pop3Server Version 0.8 (c) 1998 Computer Mutter" );
      System.out.println( "adapted 1999 by Johannes Plachy" );

      try
      {
         ConfigurationManager configurationManager = new ConfigurationManager( ConfigurationManager.getDefaultCFGFilename() );

         ErrorReporter.getInstance().updateCFG( configurationManager );

         if ( args.length == 1 )
         {
            configurationManager.setHome( args[ 0 ] );
         }

         System.out.println( configurationManager );
         startServer( configurationManager );
      }
      catch ( Exception exception )
      {
         System.err.println( exception );
      }
   }


}



/*--- formatting done in "JPS Convention" style on 08-20-2001 ---*/

