/*--- formatted by Jindent 2.1, (www.c-lab.de/~jindent) ---*/

/*
 * Copyright (c) 1999 / 2000  Johannes Plachy & Computer Mutter
 *
 * JMailSrv@jps.at
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * If you modify this file, please send us a copy.
 */

/********************************************************************************************
* CVS Info
*
* $Source: /cvsroot/jmailsrv/jmailserver/at/jps/mailserver/Sender.java,v $
*
* $Author: furykid $
* $Date: 2004/01/25 18:22:50 $
*
* modification history:
*
* $Log: Sender.java,v $
* Revision 1.5  2004/01/25 18:22:50  furykid
* bugfix 791383
* null-string got tokenized
*
* Revision 1.4  2003/05/29 18:04:47  furykid
* rebuild 290503
*
* Revision 1.3  2003/04/01 20:35:58  furykid
* cleanup
*
* Revision 1.2  2002/01/20 16:07:13  furykid
* added new header for cvs information
*

*********************************************************************************************/


/*
 * RFC 821
 *
 * possible return codes
 *
 *
 *
 * 500 Syntax error, command unrecognized
 * [This may include errors such as command line too long]
 * 501 Syntax error in parameters or arguments
 * 502 Command not implemented
 * 503 Bad sequence of commands
 * 504 Command parameter not implemented
 *
 * 211 System status, or system help reply
 * 214 Help message
 * [Information on how to use the receiver or the meaning of a
 * particular non-standard command; this reply is useful only
 * to the human user]
 *
 * 220 <domain> Service ready
 * 221 <domain> Service closing transmission channel
 * 421 <domain> Service not available,
 * closing transmission channel
 * [This may be a reply to any command if the service knows it
 * must shut down]
 *
 * 250 Requested mail action okay, completed
 * 251 User not local; will forward to <forward-path>
 * 450 Requested mail action not taken: mailbox unavailable
 * [E.g., mailbox busy]
 * 550 Requested action not taken: mailbox unavailable
 * [E.g., mailbox not found, no access]
 * 451 Requested action aborted: error in processing
 * 551 User not local; please try <forward-path>
 * 452 Requested action not taken: insufficient system storage
 * 552 Requested mail action aborted: exceeded storage allocation
 * 553 Requested action not taken: mailbox name not allowed
 * [E.g., mailbox syntax incorrect]
 * 354 Start mail input; end with <CRLF>.<CRLF>
 * 554 Transaction failed
 *
 *
 *
 */

package at.jps.mailserver;

import at.jps.nslookup.DNSRequest;

import java.io.*;
import java.net.Socket;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.StringTokenizer;
import java.util.Vector;

/**
 * Sender Send the E-Mail from the local queue to a Internet Smtp server and deletes afterward the local Messages.
 * (c) 1998 Computer Mutter computer@mutter.com adapted 1999 by Johannes Plachy
 */
final public class Sender extends Thread implements CFGUpdateable
{
   private Event                 ivThereIsMailToSend;
   private final static String   cvOutgoingMailQueueName = "Queue";
   private ConfigurationManager  ivConfigurationManager;

   // private MailMessage           ivMessages[] = null;

   private Hashtable             DNLookupList = null;
   // final private int             MAX_RETRIES = 3;
   final private static int      _RECV_TIMEOUT = 500;

   /**
    * constructs a sender thread to deliver remote mail
    * @param CFGFilename path to the configuration file
    */
   public Sender( ConfigurationManager configurationManager )
   {
      updateCFG( configurationManager );

      if ( getConfiguration().getRunSenderDaemon() )
      {
         ivThereIsMailToSend = new Event();
      }

      DNLookupList = new Hashtable( 10 );

      writeToLog( Version.getVersion( "Sender" ) );
      OutgoingMessagesManager.getInstance( configurationManager );
   }


   /**
    * method that gets called if mail is to be delivered
    */
   final public void processMail()
   {
      if ( ivThereIsMailToSend != null )
      {
         ivThereIsMailToSend.trigger();
      }
   }


   /**
    * Method declaration
    *
    *
    * @return
    *
    */
   final public static int getDefaultReceiveDelay()
   {
      return _RECV_TIMEOUT;
   }


   /**
    * name of oubound queue
    * @param name of queue
    */
   final public static String getOutgoingMailQueueName()
   {
      return cvOutgoingMailQueueName;
   }


   /**
    * get configuration information
    * @return name of queue
    */
   final private ConfigurationManager getConfiguration()
   {
      return ivConfigurationManager;
   }


   /**
    * get administrator name
    * @return name of administrator
    */
   final private String getAdmin()
   {
      return getConfiguration().getAdmin();
   }


   /**
    * read input from server reply can handle replies spanning multiple lines !
    * @return next line
    */
   final private String getServerReply( BufferedReader bufferedreader, String s ) throws Exception
   {
      String   serveResponse = null;
      String   serveResponse1 = null;

      do
      {
         serveResponse1 = null;

         try
         {
            serveResponse1 = bufferedreader.readLine();
         }
         catch ( IOException iox )
         {

            // no problem !

            serveResponse1 = null;
         }

         if ( serveResponse1 != null )
         {
            if ( serveResponse != null )
            {
               serveResponse += "\n" + serveResponse1;
            }
            else
            {
               serveResponse = serveResponse1;
            }

            if ( Debug.enabled )
            {
               writeToLog( "Server reply  : <-- " + serveResponse1 );
            }

            // rc starting with 2 or 3 are ok !

            if ( serveResponse1.charAt( 0 ) != '2' && serveResponse1.charAt( 0 ) != '3' )
            {
               Exception   exception = new Exception( s + serveResponse );

               throw exception;
            }
         }
      }
      while ( ( serveResponse1 != null ) && ( !serveResponse1.equals( "" ) ) );

      return serveResponse;
   }


   /**
    * returns number of mails
    * @return number of mails
    */
//   final private int getMsgCount( String s )
//   {
//      try
//      {
//         StringTokenizer   stringtokenizer = new StringTokenizer( s );
//         String            s1 = stringtokenizer.nextToken();
//
//         s1 = stringtokenizer.nextToken();
//
//         return Integer.parseInt( s1 );
//      }
//      catch ( Exception _ex )
//      {
//         return 0;
//      }
//   }


   /**
    * prints exception to logfile
    * @param exception to be described
    */
   final private void handleException( Throwable throwable )
   {
      String   s = throwable.toString();

      writeToLog( "Error: *** " + s );

      if ( Debug.enabled )
      {
         Thread.dumpStack();
      }

      String   msg = BugReportMsg.createExceptionAlertMessage( throwable, getConfiguration() );

      try
      {
         getConfiguration().getMessageHandler().saveMessage( getAdmin(), msg );
      }
      catch ( Exception _ex )
      {
         writeToLog( "Error: *** Errorreport could not be delivered !" );
      }
   }


   /**
    * get all messages to be send to remote servers
    */

   /*
    * final private void retrieveMsgInfo()
    * {
    * ivMessages =
    * getConfiguration().getMessageHandler().getMessages( getOutgoingMailQueueName() );
    * }
    */

   /**
    * try to retrieve the MX Record for a specified domain if no DNS server is specified or a MX Record cannot
    * be rerieved the message is send to the specified SMTP server which would do the real work for us.
    * @param ReceiverAddress is in the format JohnDoe@maildomain.com
    */
   final private DNSRequest doDNSRequest( String NameServer, String Query, int typeId )
   {
      DNSRequest  dnRequest = null;

      try
      {
         dnRequest = new DNSRequest();

         dnRequest.setMessage( Query, typeId );

         int   c = dnRequest.sendMessage( NameServer );

         if ( c == 0 )
         {
            final int   MAX_DNS_RETRIES = 10;
            int         cntr = 0;

            while ( ( ( c = dnRequest.receiveMessage() ) == -1 ) && ( cntr < MAX_DNS_RETRIES ) )
            {
               cntr++;
            }
            ;

            if ( cntr >= MAX_DNS_RETRIES )
            {
               c = -1;

               // get best mail server

            }

            if ( c == 0 )
            {
               if ( dnRequest.getReturnCode() == 3 )
               {
                  if ( Debug.enabled == true )
                  {
                     System.out.println( "Return code: destination MX servername could not be resolved :" + Query );
                  }

                  dnRequest = null;
               }
            }
            else
            {
               dnRequest = null;

               // no response

               writeToLog( "Configuration problem, please verify your DNServer settings " );
               writeToLog( "or your internet connection " );
            }
         }
         else
         {
            dnRequest = null;

            // no sending possible

            writeToLog( "Configuration problem, please verify your DNServer settings " );
         }
      }
      catch ( Exception exception )
      {
         handleException( exception );
      }

      return dnRequest;
   }


   /**
    * Method declaration
    * @param ReceiverAddress
    * @return
    * @throws IOException
    */
//   final private Socket getMXSocket( String receiverAddress ) throws IOException
//   {
//      boolean  useISPserver = false;
//      Socket   socket = null;
//      String   receiverDomain = getDomainFromAddress( receiverAddress );
//      String   mailServer = null;
//      String   namerServer = getConfiguration().getDNServer();
//
//      if ( namerServer != null )    // we can do a lookup !
//      {
//         mailServer = ( String )DNLookupList.get( receiverDomain );
//
//         if ( ( mailServer == null ) || ( mailServer.length() < 1 ) )
//         {
//            DNSRequest  dnRequest = doDNSRequest( namerServer, receiverDomain, DNSRequest.QT_MX );
//
//            if ( dnRequest != null )
//            {
//               for ( int i = 0; i < dnRequest.getNumberOfAnswers(); i++ )
//               {
//                  if ( dnRequest.getType( i ) == DNSRequest.QT_MX )
//                  {
//
//                     // this should be optimized to retrieve the best server
//                     // based on the 'preference' values returned.
//                     // by now the first record is taken.
//
//                     mailServer = dnRequest.getAnswer( i );
//                     mailServer = mailServer.substring( 0, mailServer.length() - 1 );
//
//                     break;         // we have one
//                  }
//               }
//            }
//            else
//            {
//
//               // DNS error
//
//               writeToLog( "Configuration problem, please verify your DNServer settings " );
//            }
//
//            if ( mailServer != null )
//            {
//
//               // save address for later reuse !
//
//               DNLookupList.put( receiverDomain, mailServer );
//            }
//         }
//      }
//
//      if ( mailServer == null )
//      {
//
//         // get ISP SMTP Server if no other choice
//
//         useISPserver = true;
//         mailServer = getConfiguration().getSMTPServer();
//      }
//
//      if ( mailServer != null )
//      {
//
//         // if we have a server build Socket
//
//         try
//         {
//            socket = new Socket( mailServer, SmtpServer.getDefaultSmtpPort() );
//
//            socket.setSoTimeout( getConfiguration().getSenderReplyDelay() );
//            writeToLog( "Receiver: " + receiverAddress + " via Mailserver : " + mailServer );
//         }
//         catch ( Exception exception )
//         {
//            handleException( exception );
//         }
//      }
//      else
//      {
//
//         // huston we got a problem
//
//         writeToLog( "Configuration problem, please verify your DNServer or SMTPServer settings " );
//      }
//
//      return socket;
//   }


   /**
    * Method declaration
    *
    *
    * @param receiverAddress
    *
    *
    * @return
    *
    * @throws IOException
    *
    */
   private String[] getMXServers( String receiverAddress ) throws IOException
   {
      String   receiverDomain = getDomainFromAddress( receiverAddress );
      String   namerServer = getConfiguration().getDNServer();
      String   SMTPServer = getConfiguration().getSMTPServer();
      int      availableMXservers = 0;
      String[] MXServers = null;

      if ( namerServer != null )    // we can do a lookup !
      {
         DNSRequest  dnRequest = doDNSRequest( namerServer, receiverDomain, DNSRequest.QT_MX );

         if ( dnRequest != null )
         {
            for ( int i = 0; i < dnRequest.getNumberOfAnswers(); i++ )
            {
               if ( dnRequest.getType( i ) == DNSRequest.QT_MX )
               {
                  availableMXservers++;
               }
            }

            if ( SMTPServer != null )
            {
               availableMXservers++;
            }

            MXServers = new String[ availableMXservers ];

            for ( int i = 0; i < dnRequest.getNumberOfAnswers(); i++ )
            {
               if ( dnRequest.getType( i ) == DNSRequest.QT_MX )
               {
                  String   servernameWithDot = dnRequest.getAnswer( i );

                  MXServers[ i ] = servernameWithDot.substring( 0, servernameWithDot.length() - 1 );
               }
               else
               {
                  MXServers[ i ] = null;
               }
            }
         }
      }

      /*
       * // add SMTP server of ISP for fallback
       *
       * if (SMTPServer != null)
       * {
       * if (MXServers == null)
       * {
       * MXServers = new String[1];
       * availableMXservers=1;
       * }
       *
       * MXServers[availableMXservers-1] = SMTPServer;
       *
       * }
       */

      return MXServers;
   }


   /**
    * this function does the real delivering stuff
    */
   final private void deliverOutgoingMail()
   {
      try
      {

         // retrieveMsgInfo();

         Vector   messages = getConfiguration().getMessagesManager().getMessages();

         if ( Debug.enabled == true )
         {
            ErrorReporter.getInstance().writeSenderMsg( "Sender - pending messages: " + messages.size() );
         }

         // writeToLog( "Nr Msgs found :  "+messages.size());

         if ( ( messages != null ) && ( messages.size() > 0 ) )
         {
            Vector   clonedMessages = ( Vector )messages.clone();
            Iterator iter = clonedMessages.iterator();

            while ( iter.hasNext() )
            {
               MessageInfo messageInfo = ( MessageInfo )iter.next();

               // check for timeout

               if ( messageInfo.tryAgain() )
               {
                  // Vector   receivers = messageInfo.getReceivers();

                  // MailMessage message = getConfiguration().getMessageHandler().getOutgoingMessage( messageInfo.getMessageId() );

                  if ( sendMessage( messageInfo ) )
                  {

                     // delete messageinfo too if all receivers ok
                     // there might aswell be the case that the msg itself is missing !
                     // if ( ( receivers == null ) || ( receivers.size() == 0 ) )
                     // {

                     getConfiguration().getMessagesManager().removeMessage( messageInfo );

                     // remove physical message
                     // }

                  }
               }

               iter.remove();
            }
         }
         else
         {

            // nop

         }
      }
      catch ( Exception exception )
      {
         handleException( exception );

         // after several errors breakout !

      }
   }


   /**
    * loop that checks and delivers mail to remote users
    */
   final public void run()
   {
      while ( true )
      {
         deliverOutgoingMail();

         // only wait after success !

         if ( getConfiguration().getRunSenderDaemon() )
         {
            ivThereIsMailToSend.waitForEvent();
         }
         else
         {
            break;
         }
      }
   }


   /**
    * returns the domainpart of an emailaddress from a message in the format johnDoe@somewhere.com
    * only somewhere.com is returned
    * @param EmailAdress this is a valid email in a format somwehat like <johnDoe@somewhere.com>
    * @return Domain part only
    */
   private String getDomainFromAddress( String EmailAddress )
   {
      StringTokenizer   tokenizer = new StringTokenizer( EmailAddress, "@>;" );
      String            DomainOnly = tokenizer.nextToken();

      DomainOnly = tokenizer.nextToken();

      return DomainOnly;
   }


   /**
    * get all messages to be send to remote servers
    * @return next line
    */
   final public boolean sendMessage( MessageInfo messageInfo ) throws IOException, Exception
   {
      Vector      receivers = messageInfo.getReceivers();
      MailMessage message = getConfiguration().getMessageHandler().getOutgoingMessage( messageInfo.getMessageId() );

      if ( getConfiguration().getMessageHandler().getNextMessage( message ) == false )
      {
         writeToLog( "Error: *** could not retrieve Msg from local storage" );

         return true;   // to get rid of crippled info !
      }

      // ByteArrayInputStream inputStream =   new ByteArrayInputStream( message.getMessage().getBytes() );
      // BufferedReader       messageReader = new BufferedReader( new InputStreamReader( inputStream ) );

      BufferedReader messageReader = message.getReader();

      // check if reader is available

      if ( messageReader == null )
      {

         // simulate complete delivery

         return true;
      }

      if ( Debug.enabled )
      {
         writeToLog( "Msg  : --> send msg " + message );
      }

      String   sender = messageReader.readLine();

      /*
       * String            receiver = messageReader.readLine();
       * Vector            receivers = new Vector( 10 );
       * StringTokenizer   tokenizer = new StringTokenizer( receiver, ";" );
       * while ( tokenizer.hasMoreTokens() )
       * {
       * String   newReceiver = tokenizer.nextToken();
       * receivers.add( newReceiver );
       * }
       *
       */

      Iterator iterator = receivers.iterator();

      // dummy receiver read.....

      messageReader.readLine();

      // this is the general beginning of the message
      // but the following line doent work correctly......

      messageReader.mark( ( int )message.getSize() );

      // now check if we want to send this message anonymous.... :-)
      // this block of code is not intended to be used in general

      boolean  sendMsgAnonymous = false;

      // until here ...

      boolean  completelyDelivered = false;

      // all receivers

      while ( iterator.hasNext() )
      {
         String   currentReceiver = ( String )iterator.next();

         // check for each receiver if he is located in a different domain
         //
         // try to deliver the mail 'some times' and tell sender the problem

         // int      retrycounter = 0;
         boolean  delivered = false;
         int      currentServer = 0;
         String[] availableMXServers = getMXServers( currentReceiver );

         if ( availableMXServers == null )
         {

            // we have a major problem - mail cannot be delivered anyway no servers ( bad domain?)

            BugReportMsg.createUndeliverableReplyMessgae( sender, currentReceiver, "No Server to handle send request found", messageReader, getConfiguration() );

            // remove current receiver

            iterator.remove();

            continue;
         }

         while ( ( delivered == false ) && ( currentServer < availableMXServers.length ) )
         {
            Socket                  socket = null;

            // PrintWriter

            InternalBufferedWriter  socketWriter = null;
            BufferedReader          socketReader = null;

            try
            {
               if ( availableMXServers[ currentServer ] == null )
               {
                  throw new Exception( "destination Mailserver null!" );
               }

               socket = new Socket( availableMXServers[ currentServer ], SmtpServer.getDefaultSmtpPort() );

               socket.setSoTimeout( _RECV_TIMEOUT );
               writeToLog( "-------------------------------------------------" );
               writeToLog( "Receiver: " + currentReceiver + " via Mailserver : " + availableMXServers[ currentServer ] );

               socketWriter = new InternalBufferedWriter( new PrintWriter( socket.getOutputStream() ) );
               socketReader = new BufferedReader( new InputStreamReader( socket.getInputStream() ) );

               // reset stream

               messageReader.reset();

               // communication
               // protocol startup

               if ( Debug.enabled )
               {
                  writeToLog( "Send          : --> OPEN cmd send" );
               }

               String            s = getServerReply( socketReader, "Error from OPEN cmd: " );
               String            s1 = null;

               try
               {
                  StringTokenizer   stringtokenizer = new StringTokenizer( s, " " ); 
                  s1 = stringtokenizer.nextToken();
               }
               catch ( Exception _ex )
               {
                  s1 = "NoHost";
               }

               socketWriter.println( "HELO " + s1 );
               socketWriter.flush();

               if ( Debug.enabled )
               {
                  writeToLog( "Send          : --> HELO " + s1 );
               }

               getServerReply( socketReader, "Error from HELO cmd: " );

               // send sender info

               socketWriter.println( "MAIL FROM: " + ( sendMsgAnonymous ? "\"John Doe\" <JDoe@nirvana.com>" : sender ) );
               socketWriter.flush();
               writeToLog( "Send          : --> MAIL FROM: " + sender );
               getServerReply( socketReader, "Error from MAIL FROM cmd: " );

               // this should be optimized to send all of the same domain at once
               // send receiver info

               socketWriter.println( "RCPT TO: " + currentReceiver );
               socketWriter.flush();
               writeToLog( "Send          : --> RCPT TO: " + currentReceiver );
               getServerReply( socketReader, "Error from RCPT TO cmd: " );
               socketWriter.println( "DATA" );
               socketWriter.flush();

               if ( Debug.enabled )
               {
                  writeToLog( "Send          : --> DATA cmd send" );
               }

               getServerReply( socketReader, "Error from DATA cmd: " );

               String   line = null;
               // int      linenr = 0;

               while ( ( line = messageReader.readLine() ) != null )
               {
                  socketWriter.println( line );
               }

               if ( Debug.enabled )
               {
                  writeToLog( "Send          : --> DATA send" );
               }

               getServerReply( socketReader, "Error from DATA cmd: " );
               socketWriter.println( "." );
               socketWriter.flush();

               if ( Debug.enabled )
               {
                  writeToLog( "Send          : --> . cmd send" );
               }

               getServerReply( socketReader, "Error from . cmd: " );

               // message delivered successfully
               // say goodby to server

               socketWriter.println( "QUIT" );
               socketWriter.flush();

               if ( Debug.enabled )
               {
                  writeToLog( "Send           : --> QUIT cmd send" );
               }

               getServerReply( socketReader, "Error from QUIT cmd: " );

               delivered = true;

               ErrorReporter.getInstance().writeTXMsg( ", sent, " + sender + ", " + currentReceiver + ", " + Long.toString( message.getSize() ) );

               // remove current receiver

               iterator.remove();

               if ( Debug.enabled )
               {
                  writeToLog( "Send          : delivered  OK" );
               }
            }
            catch ( Throwable t )
            {
               if ( Debug.enabled )
               {
                  writeToLog( "Send          : delivering  FAILED" );
               }

               ErrorReporter.getInstance().writeTXMsg( ", sent failed, " + sender + ", " + currentReceiver + ", " + Long.toString( message.getSize() ) );

               if ( messageInfo.getTryCount() == availableMXServers.length )
               {
                  messageInfo.setTryCount( messageInfo.getTryCount() + 1 );

                  // if retryCount is too high send msg back

                  if ( messageInfo.getTryCount() >= messageInfo.getMaxRetryCount() )
                  {
                     BugReportMsg.createUndeliverableReplyMessgae( sender, currentReceiver, "Mail could not delivered for specified receiver", messageReader, getConfiguration() );

                     // remove current receiver

                     iterator.remove();
                  }
               }

               currentServer++;
            }
            finally
            {
               if ( socketReader != null )
               {
                  socketReader.close();
               }

               if ( socketWriter != null )
               {
                  socketWriter.flush();
                  socketWriter.close();
               }

               if ( socket != null )
               {
                  socket.close();
               }
            }
         }
      }

      messageReader.close();

      if ( receivers.size() == 0 )
      {
         completelyDelivered = true;
      }

      // all receivers served

      if ( completelyDelivered )
      {

         // delete Message physically

         getConfiguration().getMessageHandler().deleteMessage( message );
      }

      return completelyDelivered;
   }


   /**
    * logging
    * @param line to be written to logfile
    */
   final private void writeToLog( String s )
   {
      ErrorReporter.getInstance().writeSenderMsg( s );
   }


   /**
    * configuration has changed
    * @param new configuration
    */
   final public void updateCFG( ConfigurationManager ConfigurationManager )
   {
      ivConfigurationManager = ConfigurationManager;

      getConfiguration().setSender( this );
   }


   /**
    * instantiate a new version and start the daemon
    * @param configuration
    */
   final public static Sender startSender( ConfigurationManager ConfigurationManager )
   {
      Sender   sender = new Sender( ConfigurationManager );

      sender.start();

      return sender;
   }


   /**
    * Method declaration
    * @param args
    */
   public static void main( String args[] )
   {
      System.out.println( Version.getVersion( "Sender" ) );

      try
      {
         ConfigurationManager configurationManager = new ConfigurationManager( ConfigurationManager.getDefaultCFGFilename() );

         ErrorReporter.getInstance().updateCFG( configurationManager );
         AutoresponderController.getInstance().updateCFG( configurationManager );

         Sender   sender = new Sender( configurationManager );

         sender.deliverOutgoingMail();
      }
      catch ( Exception exception )
      {
         System.err.println( exception );
      }
   }


}


/**
 * Class declaration
 * @author
 * @version %I%, %G%
 */
class InternalBufferedWriter extends BufferedWriter
{

   /**
    * Constructor declaration
    * @param out
    */
   public InternalBufferedWriter( Writer out )
   {
      super( out );
   }


   /**
    * Create a new buffered character-output stream that uses an output buffer of the given size.
    * @param  out  A Writer
    * @param  sz   Output-buffer size, a positive integer
    * @exception  IllegalArgumentException  If sz is <= 0
    */
   public InternalBufferedWriter( Writer out, int sz )
   {
      super( out, sz );
   }


   /**
    * Method declaration
    *
    *
    * @throws IOException
    *
    */
   public void newLine() throws IOException
   {
      write( "\r\n" );
   }


   /**
    * Write a string.
    * @param  str  String to be written
    * @exception  IOException  If an I/O error occurs
    */
   public void println( String str ) throws IOException
   {
      write( str );
      newLine();
   }


   /**
    * Method declaration
    *
    *
    * @param str
    *
    *
    * @throws IOException
    *
    */
   public void print( String str ) throws IOException
   {
      write( str );
   }


}



/*--- formatting done in "JPS Convention" style on 08-20-2001 ---*/

