/*--- formatted by Jindent 2.1, (www.c-lab.de/~jindent) ---*/

/*
 * Copyright (c) 1999 / 2000  Johannes Plachy & Computer Mutter
 *
 * JMailSrv@jps.at
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * If you modify this file, please send us a copy.
 */

/********************************************************************************************
* CVS Info
*
* $Source: /cvsroot/jmailsrv/jmailserver/at/jps/mailserver/SmtpServer.java,v $
*
* $Author: furykid $
* $Date: 2003/05/29 18:04:47 $
*
* modification history:
*
* $Log: SmtpServer.java,v $
* Revision 1.4  2003/05/29 18:04:47  furykid
* rebuild 290503
*
* Revision 1.3  2003/04/01 19:08:12  furykid
* cleanup
*
* Revision 1.2  2002/01/20 16:07:13  furykid
* added new header for cvs information
*

*********************************************************************************************/


package at.jps.mailserver;

import java.io.IOException;
import java.net.ServerSocket;

/**
 * Smtp3Server Creates an ServerSocket on a spezified port and waits for clients to be connected.
 * Then starts for every connection an Thread with an Smtp3Connection class.
 * The Server compares the host keyword from the mail.cfg configuration file
 * with the to statement from the client. Are they equal the E-Mail will be
 * local stored in the user directory. Are the not equal the E-Mail will be stored in the homeDir\Queue directory.
 * This can be run as an application !
 * @author (c) 1998/99 Computer Mutter computer@mutter.com ; Johannes Plachy
 * @version 0.99 11/99
 */
final public class SmtpServer extends Thread implements CFGUpdateable
{

   /**
    * default SMTP Port = 25
    */
   final private static int   DEFAULT_ServerSocket = 25;

   /**
    * open serversocket
    */
   private ServerSocket       ivSmtpSocket;

   /**
    * current SMTP Connection handler
    */
   private SmtpConnection     ivSmtpConnection;

   /**
    * current configuration
    */
   ConfigurationManager       ivConfigurationManager;

   /**
    * create new SMTP server with configuration data
    * @param configuration data
    */
   public SmtpServer( ConfigurationManager ConfigurationManager )
   {
      updateCFG( ConfigurationManager );

      try
      {
         setSmtpSocket( new ServerSocket( getConfiguration().getLocalSMTPServerSocket() ) );
      }
      catch ( IOException ioexception )
      {
         handleException( ioexception, "error cannot start" );

         return;     // System.exit(1);
      }

      start();
   }


   /**
    * update configuration data
    * @param new configuration data
    */
   final public void updateCFG( ConfigurationManager ConfigurationManager )
   {
      ivConfigurationManager = ConfigurationManager;
   }


   /**
    * retrieve configuration data
    * @returns configuration data
    */
   final private ConfigurationManager getConfiguration()
   {
      return ivConfigurationManager;
   }


   /**
    * returns default SMTP port
    * @returns 25
    */
   final static public int getDefaultSmtpPort()
   {
      return DEFAULT_ServerSocket;
   }


   /**
    * writes exception information to logfile
    * @param exception to be logged
    * @param more info about exception
    */
   final private void handleException( Throwable throwable, String Comment )
   {
      writeErrorMsg( throwable.toString() + " " + Comment );

      if ( Debug.enabled == true )
      {
         throwable.printStackTrace();
      }
   }


   /**
    * wait for conections and spins off SMTP connection handlers for each request
    */
   final public void run()
   {
      try
      {
         do
         {
            java.net.Socket   socket;

            try
            {
               if ( Debug.enabled )
               {
                  writeStatusMsg( "SMTP Server waiting for connection..." );
               }

               socket = getSmtpSocket().accept();
            }
            catch ( IOException ioexception )
            {
               handleException( ioexception, "error checking port: " + getConfiguration().getLocalSMTPServerSocket() );

               break;
            }

            if ( Debug.enabled )
            {
               writeStatusMsg( "SMTP Server starting new connectionhandler..." );
            }

            ivSmtpConnection = new SmtpConnection( ivConfigurationManager, socket );

            if ( ivSmtpConnection != null )
            {
               getConfiguration().getThreadPool().addTask( ivSmtpConnection );
            }
            else
            {
               writeErrorMsg( "ivSmtpConnection == null <- error" );
            }
         }
         while ( true );
      }
      catch ( Throwable exception )
      {
         handleException( exception, "Exception happened in SMTP run Loop" );
      }
   }


   /**
    * returns server socket for internal use
    * @returns open serversocket
    */
   ServerSocket getSmtpSocket()
   {
      return ivSmtpSocket;
   }


   /**
    * set new server socket for internal use
    * @param open serversocket
    */
   final private void setSmtpSocket( ServerSocket serversocket )
   {
      ivSmtpSocket = serversocket;
   }


   /**
    * write exception information to file
    * @param configuration
    */
//   final private void handleException( Throwable throwable )
//   {
//      writeErrorMsg( throwable.toString() );
//   }


   /**
    * write status message to logfile
    * @param line to be logged
    */
   final private void writeStatusMsg( String s )
   {
      ErrorReporter.getInstance().writeSmtpMsg( s );
   }


   /**
    * error message to be logged
    * @param line to be logged
    */
   final private void writeErrorMsg( String s )
   {
      ErrorReporter.getInstance().writeSmtpMsg( s, "Error" );
   }


   /**
    * utility function to instantiate a new server
    * @param coniguration data
    */
   final public static SmtpServer startServer( ConfigurationManager ConfigurationManager )
   {
      return new SmtpServer( ConfigurationManager );
   }


   /**
    * to run a standalone version of the SMTP server
    * @param hostname
    * @param homedirectoy
    */
   public static void main( String args[] )
   {
      System.out.println( "Java SmtpServer Version 0.8 (c) 1998 Computer Mutter" );
      System.out.println( "adapted 1999 by Johannes Plachy" );

      try
      {
         ConfigurationManager configurationManager = new ConfigurationManager( ConfigurationManager.getDefaultCFGFilename() );

         ErrorReporter.getInstance().updateCFG( configurationManager );
         startServer( configurationManager );
      }
      catch ( Exception exception )
      {
         System.err.println( exception );
      }
   }


}



/*--- formatting done in "JPS Convention" style on 08-20-2001 ---*/

