/*--- formatted by Jindent 2.1, (www.c-lab.de/~jindent) ---*/

/*
 * Copyright (c) 1999 / 2000  Johannes Plachy & Computer Mutter
 *
 * JMailSrv@jps.at
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * If you modify this file, please send us a copy.
 */

/********************************************************************************************
* CVS Info
*
* $Source: /cvsroot/jmailsrv/jmailserver/at/jps/mailserver/UserList.java,v $
*
* $Author: furykid $
* $Date: 2002/01/20 16:07:13 $
*
* modification history:
*
* $Log: UserList.java,v $
* Revision 1.3  2002/01/20 16:07:13  furykid
* added new header for cvs information
*

*********************************************************************************************/


package at.jps.mailserver;

import java.io.*;
import java.util.Vector;
import java.util.zip.*;

/**
 * User List manages the users of the mailserver
 * @author  Computer Mutter computer@mutter.com; Johannes Plachy
 * @version 0.99, 11/22/99
 */
final public class UserList
{

    /**
     * default filename for userinformation
     */
    final private static String   UserCFGFilename = "user.cfg";

    /**
     * holds Users
     */
    private Vector                ivVector;

    /**
     * the file that is used for serialization
     */
    private String                ivUserFile;

    /**
     * the directory where the file resides
     */
    private String                ivUserDir;

    /**
     * initializes an instance and reads userinfo from file
     * @param directory where the user.cfg file resides
     */
    public UserList( String HomeDir )
    {
        setVector( new Vector() );
        setUserDir( HomeDir );
        setUserFile( getUserDir() + File.separator + UserCFGFilename );
        readList();
    }


    /**
     * add new user to internal list
     * @param usename
     * @param password
     */
    public void addUser( String username, String password )
    {
        if ( ( username != null ) && ( password != null ) )
        {
            if ( contains( username ) )
            {
                System.out.println( "WARNING = User '" + username + "' with pwd (" + getPassword( username ) + ") existed" );
                System.out.println( "will be replaced by = User '" + username + "' with pwd (" + password + ")" );
                delUser( username );
            }

            getVector().addElement( new User( username.toLowerCase(), password.toLowerCase() ) );
            writeList();
        }
    }


    /**
     * try to find a user in the internal list
     * @param username
     * @return true if user exists
     */
    public boolean contains( String username )
    {
        boolean  result = false;

        if ( username != null )
        {
            result = getVector().contains( new User( username.toLowerCase(), "" ) );
        }

        return result;
    }


    /**
     * delete user from the internal list
     * @param usename
     */
    public boolean delUser( String username )
    {
        boolean  flag = false;

        if ( username != null )
        {
            getVector().removeElement( new User( username, "" ) );
            writeList();
        }

        return flag;
    }

    public User getUserByName(String username)
    {
        User user = null;

        if ( username != null )
        {
            int i = getVector().indexOf( new User( username, "" ) );

            if ( i < 0 )
            {
                user = null;
            }
            else
            {
                user =  ( User )getVector().elementAt( i );
            }
        }

        return user;
    }

    /**
     * get password for user from the internal list
     * @param usename
     */
    public String getPassword( String username )
    {
        String   result = null;

        User user = getUserByName( username);

        if (user != null)
        {
            result = user.getPassword().toLowerCase();
        }

        return result;

    }


    /**
     * get specific user from the internal list
     * @param index
     * @return username
     */
    public String getUser( int i )
    {
        String   result = null;

        try
        {
            User  user = ( User )getVector().elementAt( i );

            result = user.getUsername();
        }
        catch ( ArrayIndexOutOfBoundsException x )
        {
        }

        return result;
    }


    /**
     * get directory where user.cfg resides
     * @return directory name
     */
    private String getUserDir()
    {
        return ivUserDir;
    }


    /**
     * get filename of
     * @param index
     * @return username
     */
    public String getUserFile()
    {
        return ivUserFile;
    }


    /**
     * get the internal list of users
     * @return vector of User
     */
    public Vector getVector()
    {
        return ivVector;
    }


    /**
     * log exception information to file
     * @param exception to be reported
     */
    final private void handleException( Throwable throwable )
    {
        throwable.printStackTrace();
    }


    /**
     * initialize internal list of users from file
     */
    public void readList()
    {
        try
        {
            ObjectInputStream objectinputstream = new ObjectInputStream( new GZIPInputStream( new FileInputStream( getUserFile() ) ) );

            setVector( ( Vector )objectinputstream.readObject() );
            objectinputstream.close();
        }
        catch ( FileNotFoundException _ex )
        {
            System.out.println( "Create new cfg file: " + getUserFile() );
        }
        catch ( Exception exception )
        {
            try
            {

                // this is for migration purpose only !

                ObjectInputStream objectinputstream = new ObjectInputStream( new FileInputStream( getUserFile() ) );

                setVector( ( Vector )objectinputstream.readObject() );
                objectinputstream.close();

                // save compressed version

                writeList();
            }
            catch ( Exception x )
            {
                handleException( exception );
            }
        }
    }


    /**
     * change password for existing user
     * @param username
     * @param password
     */
    public void setPassword( String username, String password )
    {
        delUser( username );
        addUser( username, password );
    }


    /**
     * change directory to read configuration from
     * @param new directory
     */
    public void setUserDir( String userDir )
    {
        ivUserDir = userDir;
    }


    /**
     * change filename to read configuration from
     * @param new filename
     */
    public void setUserFile( String s )
    {
        ivUserFile = s;
    }


    /**
     * change list of internal Users
     * @param new vector
     */
    void setVector( Vector vector )
    {
        ivVector = vector;
    }


    /**
     * returns the size of the internal list
     * @return the number of entries
     */
    public int size()
    {
        return getVector().size();
    }


    /**
     * convert all entries to lowercase
     */
    public void toLowerCase()
    {
        Vector   vector = getVector();

        if ( vector != null )
        {
            for ( int i = 0; i < vector.size(); i++ )
            {
                User  user = ( User )getVector().elementAt( i );

                delUser( user.getUsername() );
                addUser( user.getPassword().toLowerCase(), user.getUsername().toLowerCase() );
            }
        }
    }


    /**
     * display internal list as string
     */
    public String toString()
    {
        String   result = null;
        Vector   vector = getVector();

        if ( vector != null )
        {
            result = getVector().toString();
        }

        return result;
    }


    /**
     * save internal list to file
     */
    void save()
    {
        writeList();
    }


    /**
     * save internal list to file
     */
    public void writeList()
    {
        try
        {
            ObjectOutputStream   objectoutputstream = new ObjectOutputStream( new GZIPOutputStream( new FileOutputStream( getUserFile() ) ) );

            objectoutputstream.writeObject( getVector() );
            objectoutputstream.close();
        }
        catch ( Exception exception )
        {
            handleException( exception );
        }
    }


    /**
     * Method declaration
     *
     *
     */
    void listAllUsers()
    {
        Vector   vector = getVector();

        if ( vector != null )
        {
            for ( int i = 0; i < vector.size(); i++ )
            {
                User  user = ( User )getVector().elementAt( i );

                System.out.println( "Username = " + user.getUsername() + "\t Pwd= " + user.getPassword() );
            }
        }
    }


    /**
     * for testing purpose only
     */
    public static void main( String args[] )
    {
        System.out.println( Version.getVersion( "UserDirectory" ) );

        if ( args.length < 3 )
        {
            UserList userlist = new UserList( "." );

            userlist.listAllUsers();
            System.out.println( "=============" );
            System.out.println( "TO ADD USERS:" );
            System.out.println( "UserList <directory> <Username> <Password>" );
        }
        else
        {
            UserList userlist = new UserList( args[ 0 ] );

            userlist.addUser( args[ 1 ], args[ 2 ] );
        }
    }


}



/*--- formatting done in "JPS Convention" style on 08-20-2001 ---*/

