/*--- formatted by Jindent 2.1, (www.c-lab.de/~jindent) ---*/

/*
 * Copyright (c) 1999 / 2000  Johannes Plachy & Computer Mutter
 *
 * JMailSrv@jps.at
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * If you modify this file, please send us a copy.
 */

/********************************************************************************************
* CVS Info
*
* $Source: /cvsroot/jmailsrv/jmailserver/at/jps/mailserver/gui/SettingsFrame.java,v $
*
* $Author: furykid $
* $Date: 2003/05/02 22:06:50 $
*
* modification history:
*
* $Log: SettingsFrame.java,v $
* Revision 1.4  2003/05/02 22:06:50  furykid
* fixed eclipse warnings
*
* Revision 1.3  2003/04/01 20:43:50  furykid
* cleanup
*
* Revision 1.2  2002/01/20 16:13:29  furykid
* added new header for cvs information
*

*********************************************************************************************/


package at.jps.mailserver.gui;

import at.jps.mailserver.ConfigurationManager;
import at.jps.mailserver.MailServerAdmin;
import at.jps.mailserver.MailServerAdminImpl;
import at.jps.mailserver.Settings;

import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.rmi.RemoteException;
import java.util.Observable;
import java.util.Observer;

import javax.swing.ImageIcon;
import javax.swing.JFrame;

/**
 * Class declaration
 * @author
 * @version %I%, %G%
 */
public class SettingsFrame extends JFrame implements Observer
{
    MailServerAdmin      ivMsAdmin = null;
    ConfigurationManager ivConfigurationManager = null;

    /**
     * Constructor declaration
     * @param msAdmin
     * @param ConfigurationManager
     */
    public SettingsFrame( MailServerAdmin msAdmin, ConfigurationManager ConfigurationManager )
    {
        HelpAboutDialog.showTimedDialog( this, "About Java Mailserver", false, 5 );

        boolean  liveConnection = false;

        ivMsAdmin = msAdmin;
        ivConfigurationManager = ConfigurationManager;

        Settings settings = null;
        String   frmTitle = "Java Mailserver Settings";

        if ( ivMsAdmin == null )
        {
            settings = ivConfigurationManager.getSettings();
        }
        else
        {
            try
            {
                settings = ivMsAdmin.getSettings();
                liveConnection = true;
            }
            catch ( RemoteException x )
            {
                System.err.println( "Remote Exception: " + x );
            }

            frmTitle = frmTitle + " -- LIVE";
        }

        if ( settings == null )
        {
            System.err.println( "Error retrieving Settings -- aborting\n" );
            System.exit( -1 );
        }

        setIconImage( new ImageIcon( getClass().getResource( "images/mailserver.gif" ), "Mailserver Icon" ).getImage() );

        setTitle( frmTitle );
        settings.addObserver( this );
        getContentPane().add( new SettingsPanel( settings, liveConnection, ivMsAdmin ) );
        pack();
        show();
        HelpAboutDialog.centerWindow( this );
    }


    /**
     * Method declaration
     * @param o
     * @param arg
     */
    public void update( Observable o, Object arg )
    {
        try
        {
            if ( ivConfigurationManager == null) // live connenction
            {
                ivMsAdmin.setSettings( ( Settings )o);

            }
            else
            {
                ivConfigurationManager.setSettings( ( Settings )o, true );
            }

        }
        catch ( Exception x )
        {
            System.err.println( "Update failed\n" );
        }
    }


    /**
     * Method declaration
     */
    private static void printHelp()
    {
        System.out.println( " Java Mailserver Configuration Tool -- Copyright Johannes Plachy 1999 \n made available to the public under the GPL licence" );
        System.out.println( "\nCmdLineOptions: -f .\\JMailSrv.cfg <- this modifies the static cfg file" );
        System.out.println( "                  -c computername    <- this can be used to dynamically modify settings \n\n" );
    }


    /**
     * Method declaration
     * @param args
     */
    public static void main( String[] args )
    {
        if ( args.length < 2 )
        {
            printHelp();

            return;
        }

        try
        {
            MailServerAdmin      msAdmin = null;
            ConfigurationManager configurationManager = null;
            String               name = args[ 1 ];

            if ( args[ 0 ].equalsIgnoreCase( "-f" ) == false )
            {
                String   computername = ( name != null ) ? name : "localhost";

                try
                {
                    msAdmin = MailServerAdminImpl.getRemoteInstance( computername );
                }
                catch ( RemoteException x )
                {
                    System.err.println( "no live connection available using static file instead\n" );
                    name = null;
                }
            }

            if ( msAdmin == null )
            {
                configurationManager = new ConfigurationManager( ( name != null ) ? name : ConfigurationManager.getDefaultCFGFilename() );
            }

            new SettingsFrame( msAdmin, configurationManager ).addWindowListener( new WindowAdapter()
                                                                                  {

                                                                                      /**
                                                                                       * Method declaration
                                                                                       *
                                                                                       *
                                                                                       * @param e
                                                                                       *
                                                                                       *
                                                                                       */
                                                                                      public void windowClosing( WindowEvent e )
                                                                                      {

                                                                                          System.exit( 0 );
                                                                                      }

                                                                                  } );

        }
        catch ( Exception x )
        {
            System.err.println( "Exception happened in SettingsFrame = " + x );
        }
    }


    /**
     * Constructor declaration
     */
    public SettingsFrame()
    {
        try
        {
            jbInit();
        }
        catch ( Exception e )
        {
            e.printStackTrace();
        }
    }


    /**
     * Method declaration
     * @throws Exception
     */
    private void jbInit() throws Exception {}


}



/*--- formatting done in "JPS Convention" style on 08-20-2001 ---*/

