/*--- formatted by Jindent 2.1, (www.c-lab.de/~jindent) ---*/

/*
 * Copyright (c) 1999 / 2000  Johannes Plachy & Computer Mutter
 *
 * JMailSrv@jps.at
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * If you modify this file, please send us a copy.
 */

/********************************************************************************************
* CVS Info
*
* $Source: /cvsroot/jmailsrv/jmailserver/at/jps/mailserver/gui/performance/BusyMonitor.java,v $
*
* $Author: furykid $
* $Date: 2002/01/20 16:13:29 $
*
* modification history:
*
* $Log: BusyMonitor.java,v $
* Revision 1.2  2002/01/20 16:13:29  furykid
* added new header for cvs information
*

*********************************************************************************************/


package at.jps.mailserver.gui.performance;

import java.awt.*;

/**
 * Class declaration
 * @author Johannes Plachy
 * @version 1 / 2000
 */
public class BusyMonitor extends Canvas implements Runnable
{
   static final int     sleepTime = 100;
   static final Color   bg = SystemColor.menu;
   static final Color   fg = SystemColor.menuText;
   static final Color   green = Color.green;
   static final Color   yellow = Color.yellow;
   static final Color   red = Color.red;
   static final Color   darkGray = Color.darkGray;
   public static int    prioMin = Thread.MIN_PRIORITY;
   public static int    prioMax = Thread.MAX_PRIORITY;
   MonitorThread[]      threads;
   Image                offscreen;

   /**
    * Constructor declaration
    */
   public BusyMonitor()
   {
      setSize( getPreferredSize() );
      setBackground( bg );
      setForeground( fg );

      threads = new MonitorThread[ prioMax - prioMin + 1 ];

      for ( int pri = prioMin, ix = 0; pri <= prioMax; pri++, ix++ )
      {
         threads[ ix ] = new MonitorThread( pri );

         threads[ ix ].start();
      }

      Thread   monitor = new Thread( this );

      monitor.setDaemon( true );
      monitor.start();
   }


   /**
    * Method declaration
    */
   public void run()
   {
      while ( true )
      {
         updateThreadMonitors();

         try
         {
            Thread.sleep( sleepTime );
         }
         catch ( Exception e ) {}
      }
   }


   /**
    * Method declaration
    */
   void updateThreadMonitors()
   {
      if ( offscreen == null )
      {
         Dimension   sz = getPreferredSize();

         offscreen = createImage( sz.width, sz.height );

         if ( offscreen == null )
         {
            return;
         }
      }

      int      x = border + gap;
      int      y = border + gap;
      Graphics g = offscreen.getGraphics();

      if ( g != null )
      {
         try
         {
            Dimension   size = getSize();

            g.setColor( getBackground() );
            g.fillRect( 0, 0, size.width, size.height );

            for ( int i = 0; i < threads.length; i++ )
            {
               MonitorThread  thread = threads[ i ];
               int            slack = ( int )thread.slack();

               if ( slack > 1000 )
               {
                  slack = 1000;
               }

               int   len = barLen * ( 1000 - slack ) / 1000;
               Color color = ( slack <= 333 ) ? green : ( slack <= 667 ) ? yellow : red;

               g.setColor( darkGray );
               g.fillRect( x, y, barLen, barWid );
               g.setColor( color );
               g.fillRect( x, y, len, barWid );

               y += barWid + gap;
            }

            Graphics screen = getGraphics();

            if ( screen != null )
            {
               try
               {
                  screen.drawImage( offscreen, 0, 0, null );
               }
               finally
               {
                  screen.dispose();
               }
            }
         }
         finally
         {
            g.dispose();
         }
      }
   }


   /**
    * Method declaration
    * @return
    */
   int threadCount()
   {
      return prioMax - prioMin + 1;
   }


   int   gap = 4;
   int   border = 2;
   int   barLen = 32;
   int   barWid = 4;

   /**
    * Method declaration
    * @return
    */
   public Dimension getPreferredSize()
   {
      int   n = threadCount();
      int   w = barLen + 2 * border + 2 * gap;
      int   h = barWid * n + 2 * border + ( n + 1 ) * gap;

      return new Dimension( w, h );
   }


   /**
    * Method declaration
    * @param g
    */
   public void update( Graphics g )
   {
      paint( g );
   }


   /**
    * Method declaration
    * @param g
    */
   public void paint( Graphics g )
   {
      Dimension   size = getSize();

      g.setColor( getBackground() );
      g.fillRect( 0, 0, size.width, size.height );
   }


   /**
    * Class declaration
    * @author
    * @version %I%, %G%
    */
   class MonitorThread extends Thread
   {
      final int   sleepTime = 10;
      long        lastUpdated = -1;
      int         priority = Thread.NORM_PRIORITY;

      /**
       * Constructor declaration
       * @param priority
       */
      MonitorThread( int priority )
      {
         this.priority = priority;

         setDaemon( true );
         setPriority( priority );
         setName( "PerfCheck " + priority );
      }


      /**
       * Method declaration
       * @return
       */
      int slack()
      {
         return ( int )( System.currentTimeMillis() - lastUpdated );
      }


      /**
       * Method declaration
       */
      public void run()
      {
         while ( true )
         {
            lastUpdated = System.currentTimeMillis();

            try
            {
               sleep( sleepTime );
            }
            catch ( Exception e ) {}
         }
      }


   }

}



/*--- formatting done in "JPS Convention" style on 08-20-2001 ---*/

