/*--- formatted by Jindent 2.1, (www.c-lab.de/~jindent) ---*/

/*
 * Copyright (c) 1999 / 2000  Johannes Plachy & Computer Mutter
 *
 * JMailSrv@jps.at
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * If you modify this file, please send us a copy.
 */

/********************************************************************************************
* CVS Info
*
* $Source: /cvsroot/jmailsrv/jmailserver/at/jps/mailserver/gui/performance/MemoryMonitor.java,v $
*
* $Author: furykid $
* $Date: 2002/01/20 16:13:29 $
*
* modification history:
*
* $Log: MemoryMonitor.java,v $
* Revision 1.2  2002/01/20 16:13:29  furykid
* added new header for cvs information
*

*********************************************************************************************/


package at.jps.mailserver.gui.performance;

import java.awt.*;
import java.awt.event.*;

/**
 * Class declaration
 * @author
 * @version %I%, %G%
 */
public class MemoryMonitor extends Canvas implements Runnable, MouseListener
{
   static Runtime       runtime = Runtime.getRuntime();
   static int           sleepTime = 100;
   static final Color   bg = SystemColor.menu;
   static final Color   fg = SystemColor.menuText;
   static final Color   totalColor = Color.blue;
   static final Color   usedColor = Color.yellow;
   static final Color   totalShadowColor = totalColor.darker().darker();
   static final Color   usedShadowColor = usedColor.darker().darker();
   Font                 labelfont = new Font( "SansSerif", Font.PLAIN, 11 );
   private Image        offscreen;
   int                  gap = 5;
   int                  border = 2;
   int                  grafW = 60;
   int                  grafH = 48;
   int                  shadowDepth = 5;

   /**
    * Constructor declaration
    */
   public MemoryMonitor()
   {
      setFont( labelfont );
      setSize( getPreferredSize() );
      setBackground( bg );
      setForeground( fg );

      Thread   monitor = new Thread( this );

      monitor.setDaemon( true );
      monitor.start();
      addMouseListener( this );
   }


   /**
    * Method declaration
    * @param f
    */
   public void setFont( Font f )
   {
      super.setFont( f );

      labelfont = f;
   }


   /**
    * Method declaration
    * @return
    */
   public Font getFont()
   {
      return labelfont;
   }


   // ---------- MouseListener Implementation ------------------

   /**
    * Method declaration
    * @param e
    */
   public void mouseEntered( MouseEvent e ) {}


   /**
    * Method declaration
    * @param e
    */
   public void mouseExited( MouseEvent e ) {}


   /**
    * Method declaration
    * @param e
    */
   public void mousePressed( MouseEvent e ) {}


   /**
    * Method declaration
    * @param e
    */
   public void mouseReleased( MouseEvent e ) {}


   /**
    * Method declaration
    * @param e
    */
   public void mouseClicked( MouseEvent e )
   {
      runtime.gc();
   }


   long  dialogDelay = 500;

   /**
    * Method declaration
    */
   public void removeNotify()
   {
      super.removeNotify();

      if ( offscreen != null )
      {
         offscreen.flush();

         offscreen = null;
      }
   }


   /**
    * Method declaration
    */
   public void run()
   {
      while ( true )
      {
         updateMemoryMonitor();

         try
         {
            Thread.sleep( sleepTime );
         }
         catch ( Exception e ) {}
      }
   }


   // bytes, Kilobytes, etc... suffix for label.

   static final private char[]   suffixes = "bKMGTP".toCharArray();

   // cache some stuff to avoid allocation each time thru loop

   private char[]                label = new char[ 32 ];    // holds the label value
   private int                   labelcount = 0;         // # valid chars in label
   private char[]                buf = new char[ 32 ];   // temp work buffer
   private Dimension             size;                   // size of display area
   private FontMetrics           fm;                     // metrics of label font
   private int[]                 widths;     // cache char widths to measure label

   // convert number bytes to a string rep, with appropriate suffix.
   // scale to 4 significant digits:  rolls over to next magnitude after
   // 1..8191, 8K..8191K, 8M..8191M, etc...

   /**
    * Method declaration
    * @param amt
    */
   private void memstring( long amt )
   {
      int   order = 0;

      while ( amt >= 8192 )
      {
         amt += 512;
         amt >>= 10;
         ++order;
      }

      int   i = ( int )amt;
      int   len = 0;

      while ( i >= 10 )
      {
         buf[ len++ ] = ( char )( '0' + i % 10 );
         i /= 10;
      }

      buf[ len++ ] = ( char )( '0' + i % 10 );

      for ( int j = len - 1; j >= 0; j-- )
      {
         label[ labelcount++ ] = buf[ j ];
      }

      label[ labelcount++ ] = suffixes[ order ];
   }


   // label is:  AVAIL / TOTAL.  (543K / 1024K).

   /**
    * Method declaration
    * @param free
    * @param total
    */
   private void buildLabel( long free, long total )
   {
      labelcount = 0;

      memstring( total - free );

      label[ labelcount++ ] = '/';

      memstring( total );
   }


   /**
    * Method declaration
    */
   void updateMemoryMonitor()
   {
      if ( offscreen == null )
      {
         size = getPreferredSize();
         offscreen = createImage( size.width, size.height );

         if ( offscreen == null )
         {
            return;
         }

         fm = getFontMetrics( labelfont );
         widths = fm.getWidths();
      }

      Graphics imagegraphic = offscreen.getGraphics();

      if ( imagegraphic != null )
      {
         try
         {
            imagegraphic.setColor( bg );
            imagegraphic.fillRect( 0, 0, size.width, size.height );

            long  total = runtime.totalMemory();
            long  free = runtime.freeMemory();
            int   arc = ( int )( 360 * ( total - free ) / total );
            int   x = border + gap;
            int   y = border + gap;

            imagegraphic.setColor( totalShadowColor );
            imagegraphic.fillOval( x + shadowDepth, y + shadowDepth, grafW, grafH );
            imagegraphic.setColor( usedShadowColor );
            imagegraphic.fillArc( x + shadowDepth, y + shadowDepth, grafW, grafH, 90, -arc );
            imagegraphic.setColor( totalColor );
            imagegraphic.fillOval( x, y, grafW, grafH );
            imagegraphic.setColor( usedColor );
            imagegraphic.fillArc( x, y, grafW, grafH, 90, -arc );

            y += gap + grafH + shadowDepth;
            x += shadowDepth / 2;

            // ***** draw the label centered in the button

            if ( labelfont != null )
            {
               buildLabel( free, total );
               imagegraphic.setColor( fg );

               int   txtW = 0;

               for ( int i = 0; i < labelcount; i++ )
               {
                  txtW += widths[ label[ i ] ];
               }

               x += grafW / 2 - txtW / 2;

               imagegraphic.drawChars( label, 0, labelcount, x, y + fm.getMaxAscent() );
            }

            Graphics screengraphic = getGraphics();

            if ( screengraphic != null )
            {
               try
               {
                  screengraphic.drawImage( offscreen, 0, 0, null );
               }
               finally
               {
                  screengraphic.dispose();
               }
            }
         }
         finally
         {
            imagegraphic.dispose();
         }
      }
   }


   /**
    * Method declaration
    * @return
    */
   public Dimension getPreferredSize()
   {
      int   w = grafW + shadowDepth + 2 * border + 2 * gap;
      int   h = grafH + shadowDepth + 2 * border + 3 * gap;

      if ( getFont() != null )
      {
         FontMetrics fm = getFontMetrics( getFont() );

         w = Math.max( w, fm.stringWidth( "XXXK / XXXM" ) + 2 * border + 2 * gap );
         h += fm.getHeight();
      }

      return new Dimension( w, h );
   }


   /**
    * Method declaration
    * @param g
    */
   public void update( Graphics g )
   {
      paint( g );
   }


   /**
    * Method declaration
    * @param g
    */
   public void paint( Graphics g )
   {
      Dimension   size = getSize();

      g.setColor( getBackground() );
      g.fillRect( 0, 0, size.width, size.height );
   }


}



/*--- formatting done in "JPS Convention" style on 08-20-2001 ---*/

