/********************************************************************************************
* CVS Info
*
* $Source: /cvsroot/jmailsrv/jmailserver/at/jps/threadpool/ThreadPool.java,v $
*
* $Author: furykid $
* $Date: 2002/01/20 16:13:29 $
*
* modification history:
*
* $Log: ThreadPool.java,v $
* Revision 1.2  2002/01/20 16:13:29  furykid
* added new header for cvs information
*

*********************************************************************************************/

package at.jps.threadpool;

/**
 */
public class ThreadPool
{
    final public static int MAX_THREADS = 10;

    /**
     * the list of pending tasks waiting to be served
     */
    private Queue tasks = new Queue();

    /**
     * the global exceptionhandler for unhandled exceptions
     */
    private ExceptionHandler exceptionHandler = null;

    private ThreadPoolGroup threadPoolGroup = null;

    /**
     * our local threadgroup primarily used for consequent
     * unhandled exeption reporting
     */
    class ThreadPoolGroup extends ThreadGroup
    {

        public ThreadPoolGroup()
        {
            super("Threadpool");
        }

        /**
         * called from the system if a unhandled exception occures
         * we simply delegate the obtained information to the special
         * ExceptionHandler
         *
         * @param t      Thread in which the exception occurred
         * @param e      Throwable is the pending Exception
         * @see ExceptionHandler
         */
        public void uncaughtException(Thread t, Throwable e)
        {
            // we try to report the pending unhandled exception to
            // a central service
            // if none is available we create our own local default handler

            // if we get here we got a severe problem which should be
            // reported but needs special handling
            // one sideeffect is that even if the JVM is in a stable condition
            // and could continue with another task
            // one of our workerthreads has been killed !

            if (getExceptionHandler() == null)
            {
                setExceptionHandler( new DefaultExceptionHandler());
            }

            getExceptionHandler().reportException( t, e);


            //super.uncaughtException(t, e);
        }
    }


    /**
     * construct a new pool with a given number of
     * workerthreads
     *
     * @param ThreadCount number of workerthreads to preallocate
     */
    public ThreadPool( int ThreadCount)
    {
        threadPoolGroup = new ThreadPoolGroup();

        for (int i = 0; i < ThreadCount;i++)
        {
            ThreadPoolThread worker = new ThreadPoolThread( threadPoolGroup, this);

            worker.start();
        }
    }



    /**
     * add a new task to the internal queue which will be
     * processed as soon as a workerthread gets idle
     *
     * @param task   a task to be processed
     */
    public void addTask( Runnable task)
    {
        tasks.put( task);
    }


    /**
     * this method returns a task if available from the
     * internal queue
     *
     * @return task
     */
    public Runnable getNextTask()
    {
        return (Runnable) tasks.get();
    }

    /**
     * initialize local exceptionhandler
     *
     * @param exceptionHandler
     *               new exceptionhandler to use
     * @see ExceptionHandler
     */
    public void setExceptionHandler(ExceptionHandler exceptionHandler )
    {
        this.exceptionHandler = exceptionHandler;
    }


    /**
     * retrieve local exceptionhandler
     *
     * @return ExceptionHandler
     * @see ExceptionHandler
     */
    public ExceptionHandler getExceptionHandler()
    {
        return this.exceptionHandler;
    }


}
